#pragma region "INCLUDES & NAMESPACE"

#include <stdio.h>					// Visual C Studio includes.
#include <stdlib.h>					// Visual C Studio includes.
#include <math.h>					// Math library.
#include <time.h>					// Time.
#include <iostream>					// Input output.
#include <fstream>					// Input output (fstream).
using namespace std;				// Input output.

#pragma endregion

#pragma region "DOCUMENTATION"

// SRCPSP List Policy Solver
// Author: Creemers Stefan
// Initiation date: 12/08/2014

// General Information:
//
// SRCPSP = Stochastic Resource-Constrained Project Scheduling Problem.
// List Policy = priority list of which activities to execute first.
//
// Input of the function:
//	- Project network.
//	- Priority list of activities that defines which activity has to be executed first if a choice has to be made.
//	- List of start-start constraints that defines which activities have to be started together.
//
// Output:
//	- Expected duration.
//	- Expected NPV.
//
// Assumptions:
//	- Activity durations have exponential durations.
//
// Note that it is also possible to extend this method to work with phase-type distributed activity durations.

// Modeling approach:
//
// The state of the system is given by the status of the individual activities.
// All possible states form a Markov Chain.
// At the completion of an activity, a decision has to be made whether or not to start an activity => we have a Markov Decision Process.
//
// A state is represented in a binary way.
// A first binary number represents which activities have been finished.
// A second binary number represents which activities are busy.
// A third binary number represents which activities are idle.
//
// In order to be efficient with respect to memory (our experience shows that memory rather than CPU time is the biggest bottleneck), we do not store the entire state space. 
// In memory we store all states where x activities have been finished & all states where x+1 activities are finished.
// Therefore, we only need two arrays of data. Once all states where x activities have been finished have been processed, the corresponding information is no longer required, and we can use the array to store the information of all states where x+2 activities have been finished.

#pragma endregion

void main()
{

	#pragma region "[STEP -1] DEFINE LOCAL VARIABLES"

	// These are general local variables that are used in different steps of the function.

	double lv_dur;	// Holds a duration.
	double lv_dur2;	// Holds a duration.
	double lv_prob;	// Holds a probability.
	double lv_prob2;	// Holds a probability.
	double lv_prob3;	// Holds a probability.
	double lv_prob4;	// Holds a probability.
	double lv_rxu;	// Holds the resource use.
	int lv_act;	// Holds an activity.
	int lv_act2;	// Holds an activity.
	int lv_act_done;	// Holds how many activities have been processed.
	int lv_bin;	// Holds a bin.
	int lv_bin_val;	// Holds a binary value.
	int lv_eli;	// Holds an eligible activity.
	int lv_eli_new;	// Holds a new eligible activity.
	int lv_nre_new;	// Holds the number of new eligible activities.
	int lv_pos;	// Holds the position of an element in the queue (used to determine transitive structures).
	int lv_pre;	// Holds a predecessor activity.
	int lv_priority;	// Holds the priority score of an existing eligible activity.
	int lv_priority_new;	// Holds the priority score of a new eligible activity.
	int lv_size; // Holds the size of the queue (used to determine transitive structures).
	int lv_suc;	// Holds a successor activity.
	int lv_suc2;	// Holds a successor activity.
	bool lv_bool;	// Boolean variable.
	bool lv_new;	// Holds whether or not a condition is met.

	int lv_b;	// Counter (bin/bit).
	int lv_c;	// Counter (SS pairs).
	int lv_e;	// Counter (eligible activities).
	int lv_e2;	// Counter (eligible activity).
	int lv_i;	// Counter (general purpose).
	int lv_j;	// Counter (activities).
	int lv_j2;	// Counter (activities).
	int lv_j3;	// Counter (activities).
	int lv_n;	// Counter (busy activity).
	int lv_n2;	// Counter (busy activity).
	int lv_n3;	// Counter (busy activity).
	int lv_m;	// Counter (bin member).
	int lv_p;	// Counter (activities).
	int lv_r;	// Counter (resources).
	int lv_s;	// Counter (successors).
	int lv_s2;	// Counter (successors).
	int lv_x;	// Counter (pivot (to be situation)).
	int lv_x2;	// Counter (pivot).
	int lv_z;	// Counter (state).
	int lv_z2;	// Counter (state).

	#pragma endregion

	#pragma region "[STEP 0] DETERMINE USER INPUT (BASIC PROJECT INFORMATION)"

	// The user has to supply the function with the following structures:
	//
	//	- double ** lm_rxu.		[lv_j][lv_r].	Holds the use of resource lv_r by activity lv_j.
	//	- double * la_dur.		[lv_j].			Holds the duration of activity lv_j.
	//	- double * la_rxa.		[lv_r].			Holds the resource availability of resource lv_r.
	//	- int lv_nra.							Holds the number of activities.
	//	- int lv_nrr.							Holds the number of resources.

	#pragma region "DEFINE STRUCTURES"

	double ** lm_rxu;	// [lv_j][lv_r]. Use of resource lv_r by activity lv_j.
	int ** lm_suc_ini;	// [lv_j][lv_s]. Successor lv_s of activity lv_j (does not yet contain additional FS precedence relations).

	double * la_dur;	// [lv_j]. Expected duration of activity lv_j.
	double * la_rxa;	// [lv_r]. Resource availability of resource lv_r.
	int * la_nrs_ini;	// [lv_j]. Number of successors for activity lv_j (does not yet contain additional FS precedence relations).

	int lv_nra;	// Number of activities.
	int lv_nrr;	// Number of resources.

	// Note: activity 0 has to represent the dummy-start activity.

	#pragma endregion

	// The code below serves the purpose of reading a random project file => this code is not required if the user provides the input himself.

	#pragma region "READ RANDOM PROJECT FILE IN ORDER TO OBTAIN BASIC PROJECT INFORMATION"

	#pragma region "[NOT REQUIRED IN FUNCTION] LOCALS REQUIRED FOR READING THE PROJECT FILE"

	int buffer_base;	// Base of the input buffer.
	int lv_J120;	// Holds whether to use the J120 dataset or the J30 dataset.
	int lv_project;	// Holds the project chosen by the user.

	char buffer [10];

	buffer_base=10;	// Initialize base of input buffer.

	#pragma endregion

	#pragma region "[NOT REQUIRED IN FUNCTION] GET USER INPUT"

	cout << "Use J30 dataset (0) or J120 dataset (1):" << "\n";
	cin >> lv_J120;
	if(lv_J120==0)
	{
		cout << "Choose a project of the J30 dataset (1-480):" << "\n";
	}
	else
	{
		cout << "Choose a project of the J120 dataset (1-600):" << "\n";
	}
	cin >> lv_project;

	#pragma endregion

	#pragma region "[NOT REQUIRED IN FUNCTION] READ PROJECT FILE"

	std::string PJFile;	// Set project file.
	if(lv_J120==0)
	{
		PJFile = "PROJECTS_J30\\J30_";	// Set the project filename.
	}
	else
	{
		PJFile = "PROJECTS_J120\\J120_";	// Set the project filename.
	}
	_itoa_s(lv_project,buffer,buffer_base);
	PJFile.append(buffer);	// Append the correct number.
	PJFile.append(".RCP");	// Append the project filename extension.
	ifstream inFile;
	inFile.open(PJFile.c_str());	// Path of stream = string fileName.
	if(!inFile)
	{
		int lv_error;
		cout << "\n";
		cout << "Unable to open project file number:" << lv_project << ".";
		cout << "\n";
		cout << "Press any key to quit...";
		cin >> lv_error;
		throw 0;
	}
	inFile >> lv_nra;	// Read the number of activities.
	lm_rxu = new double * [lv_nra];
	lm_suc_ini = new int * [lv_nra];
	la_dur = new double [lv_nra];
	la_nrs_ini = new int [lv_nra];
	inFile >> lv_nrr;	// Read the number of resources.
	la_rxa = new double [lv_nrr];
	for(lv_r=0;lv_r<lv_nrr;lv_r++)
	{
		inFile >> la_rxa[lv_r];	// Read the resource availability of resource lv_r.
	}
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		lm_rxu[lv_j] = new double [lv_nrr];
		lm_suc_ini[lv_j] = new int [lv_nra];
		inFile >> la_dur[lv_j];	// Read the expected duration of activity lv_j.
		for(lv_r=0;lv_r<lv_nrr;lv_r++)
		{
			inFile >> lm_rxu[lv_j][lv_r];	// Read the the use of resource lv_r by activity lv_j.
		}
		inFile >> la_nrs_ini[lv_j];	// Read the number of successors of activity lv_j.
		lm_suc_ini[lv_j] = new int [la_nrs_ini[lv_j]];
		for(lv_s=0;lv_s<la_nrs_ini[lv_j];lv_s++)
		{
			inFile >> lm_suc_ini[lv_j][lv_s];	// Read successor lv_s of activity lv_j.
			lm_suc_ini[lv_j][lv_s]--;	// Decrement successor number in order to make sure that 0 is the dummy-start activity.
		}
	}
	inFile.close();	// Close the file.

	#pragma endregion

	#pragma endregion

	#pragma endregion

	#pragma region "[STEP 1] SET SYSTEM PARAMETERS"

	// Defines the system parameters of the function. These parameters control the memory and CPU requirement of the function. They determine whether or not a feasible solution can be found.

	int lv_bits;	// Holds the size of a bin/the maximum number of bin members. Default = 16.
	int lv_mxp;	// Holds the maximum number of activities that are allowed to be executed in parallel/that are eligible to start.
	int lv_mxz;	// Holds the maximum number of states for each pivot element.

	lv_bits=16;	// Set the size of a bin/the maximum number of bin members.
	lv_mxp=24;	// Set the maximum number of activities that are allowed to be executed in parallel/that are eligible to start.
	lv_mxz=100000;	// Set the maximum number of states for each pivot element.

	#pragma endregion
	
	#pragma region "[STEP 2] INITIALIZED STRUCTURES (INITIALIZED BEFORE THE FIRST FUNCTION CALL)"

	// Creates & initializes structures that have to be initialized only once, before the first function call.
	// Information that has to be available at this point:
	//	- double * la_dur.	[lv_j].	(basic project information).	Expected duration of activity lv_j.
	//	- int lv_bits.				(system parameter).				Holds the size of a bin/the maximum number of bin members. Default = 16.
	//	- int lv_nra.				(basic project information).	Number of activities.

	#pragma region "DEFINE STRUCTURES"

	int ** lm_bin_act;	// [lv_i][lv_m]. Holds the lv_m^th activity in bin lv_i.

	double * la_rate;	// [lv_j]. Holds the exponential rate (1 divided by the expected duration) of activity lv_j.
	int * la_bin;	// [lv_b]. Holds 2^lv_b at position lv_b.
	int * la_bin_act;	// [lv_j]. Holds the bin of activity lv_j.
	int * la_bin_nra;	// [lv_i]. Holds the number of activities in bin lv_i.
	int * la_bin_val_act;	// [lv_j]. Holds the binary value that represents activity lv_j.

	int lv_mxb_size;	// Holds the maximum bin size.
	int lv_nrb;	// Holds the number of bins required to capture lv_nra activities.

	#pragma endregion

	#pragma region "CREATE & INITIALIZE BASIC BINARY STRUCTURES (lm_bin2act, la_bin, la_bin_act, la_bin_act_val, la_bin_nra, lv_mxb_size, & lv_nrb)"

	la_bin = new int [lv_bits+1];
	la_bin[0]=1;	// Initialize first entry.
	lv_mxb_size=1;	// Initialize maximum bin size.
	for(lv_b=1;lv_b<=lv_bits;lv_b++)
	{
		la_bin[lv_b]=(la_bin[(lv_b-1)]*2);	// Set entry lv_b.
		lv_mxb_size=(lv_mxb_size*2);	// Set new maximum bin size.
	}
	lm_bin_act = new int * [lv_mxb_size+1];
	la_bin_nra = new int [lv_mxb_size+1];
	for(lv_i=0;lv_i<=lv_mxb_size;lv_i++)
	{
		la_bin_nra[lv_i]=0;	// Initialize the number of activities in bin lv_i.
		for(lv_b=0;lv_b<=lv_bits;lv_b++)
		{
			if((lv_i & la_bin[lv_b])==la_bin[lv_b])	// lv_b is a member of bin lv_i.
			{
				la_bin_nra[lv_i]++;	// Increment the number of activities in bin lv_i.
			}
		}
		lm_bin_act[lv_i] = new int [la_bin_nra[lv_i]];
		la_bin_nra[lv_i]=0;	// Initialize the number of activities in bin lv_i.
		for(lv_b=0;lv_b<=lv_bits;lv_b++)
		{
			if((lv_i & la_bin[lv_b])==la_bin[lv_b])	// lv_b is a member of bin lv_i.
			{
				lm_bin_act[lv_i][la_bin_nra[lv_i]]=lv_b;	// Record lv_b as a member of bin lv_i.
				la_bin_nra[lv_i]++;	// Increment the number of activities in bin lv_i.
			}
		}
	}
	lv_nrb=static_cast<int>(ceil(static_cast<double>(lv_nra)/static_cast<double>(lv_bits)));	// Get the number of bins required.
	la_bin_act = new int [lv_nra];
	la_bin_val_act = new int [lv_nra];
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		la_bin_act[lv_j]=static_cast<int>(floor(static_cast<double>(lv_j)/static_cast<double>(lv_bits)));	// Get correct bin.
		la_bin_val_act[lv_j]=la_bin[lv_j-(la_bin_act[lv_j]*lv_bits)];	// Get the correct bin value.
	}

	#pragma endregion

	#pragma region "CREATE & INITIALIZE EXPONENTIAL ACTIVITY DURATION RATES (la_rate)"

	la_rate = new double [lv_nra];
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		if(la_dur[lv_j]>0)
		{
			la_rate[lv_j]=(1.0/la_dur[lv_j]);	// Get the exponential rate of activity lv_j.
		}
		else
		{
			la_rate[lv_j]=0;	// Get the exponential rate of activity lv_j.
		}
	}

	#pragma endregion

	#pragma endregion

	#pragma region "[STEP 3] EMPTY STRUCTURES (INITIALIZED DURING EACH FUNCTION CALL)"

	// Creates structures that have to be initialized during each function call.
	// Information that has to be available at this point:
	//	- int lv_mxp.	(system parameter).				Holds the maximum number of activities that are allowed to be executed in parallel/eligible to start.
	//	- int lv_mxz.	(system parameter).				Holds the maximum number of states for each pivot element.
	//	- int lv_nra.	(basic project information).	Number of activities.
	//	- int lv_nrb.	(obtained from STEP 2).			Holds the number of bins required to capture lv_nra activities.
	//	- int lv_nrr.	(basic project information).	Number of resources.

	#pragma region "DEFINE STRUCTURES"

	double *** lm_state_rxa;	// [lv_x][lv_z][lv_r]. Holds the availability of resource lv_r in state lv_z for pivot lv_x.
	int *** lm_state_bin_fin;	// [lv_x][lv_z][lv_b]. For pivot lv_x holds the binary representation of the finished activities in state lv_z.
	int *** lm_state_bin_idl;	// [lv_x][lv_z][lv_b]. For pivot lv_x holds the binary representation of the idle activities in state lv_z.
	int *** lm_state_bus;	// [lv_x][lv_z][lv_m]. For pivot lv_x holds the lv_m^th ongoing activity in state lv_z.
	int *** lm_state_eli;	// [lv_x][lv_z][lv_e]. For pivot lv_x holds the lv_e^th eligible activity in state lv_z. Eligble activities are ranked according to their position in the priority list.
	bool *** lm_state_eli_act;	// [lv_x][lv_z][lv_j]. Holds whether activity lv_j is eligible to start or not for pivot lv_x and state lv_z.
	
	double ** lm_state_makespan;	// [lv_x][lv_z]. Holds the makespan for state lv_z at pivot lv_x.
	double ** lm_state_prob;	// [lv_x][lv_z]. Holds the probability to visit state lv_z at pivot lv_x.
	double ** lm_state_tot_rate;	// [lv_x][lv_z]. Holds the total rate of the ongoing activities in state lv_z for pivot lv_x.
	int ** lm_aSS_bin;	// [lv_j][lv_b]. Holds the binary representation of all pair members that need to be busy/finished in order to be able to start activity lv_j.
	int ** lm_aSS_mem;	// [lv_j][lv_m]. Holds the lv_m^th SS pair member of activity lv_j (all pair members have to be completed/ongoing before activity lv_j can be started).
	int ** lm_bin_tpre;	// [lv_j][lv_b]. Holds the binary representation of all transitive predecessors of activity lv_j.
	int ** lm_pre;	// [lv_j][lv_p]. Predecessor lv_p of activity lv_j.
	int ** lm_state_nrb;	// [lv_x][lv_z]. For pivot lv_x holds the number of activities that are ongoing in state lv_z.
	int ** lm_state_nre;	// [lv_x][lv_z]. For pivot lv_s holds the number of eligible activities in state lv_z.
	int ** lm_suc_ranked;	// [lv_j][lv_s]. Holds the lv_s^th ranked successor of activity lv_j.
	int ** lm_tpre;	// [lv_j][lv_p]. Transitive predecessor lv_p of activity lv_j.

	int * la_aSS_nrm;	// [lv_j]. Holds the number of SS pair members of activity lv_j (all pair members have to be completed/ongoing before activity lv_j can be started).
	int * la_act_priority;	// [lv_j]. Holds the priority score of activity lv_j (high score = high priority).
	int * la_bin_fin;	// [lv_b]. Holds the binary code of all finished activities.
	int * la_nrp;	// [lv_j]. Number of predecessors for activity lv_j.
	int * la_nrtp;	// [lv_j]. Number of transitive predecessors for activity lv_j.
	int * la_queue;	// [lv_i]. Holds queue element lv_i (used to determine transitive structures).
	int * la_state_nrz;	// [lv_x]. For pivot lv_x holds the number of states.

	int lv_mxp_SS;	// Holds the maximum number of SS pairs.

	#pragma endregion

	#pragma region "CALCULATE MAXIMUM NUMBER OF SS PAIRS (lv_mxp_SS)"

	lv_mxp_SS=(lv_nra*lv_nra);	// Get the maximum number of SS pairs.

	#pragma endregion

	#pragma region "CREATE SS PAIR STRUCTURES (lm_aSS_bin, lm_aSS_mem, & la_aSS_nrm)"

	lm_aSS_bin = new int * [lv_nra];
	lm_aSS_mem = new int * [lv_nra];
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		lm_aSS_bin[lv_j] = new int [lv_nrb];
		lm_aSS_mem[lv_j] = new int [lv_nra];
	}
	la_aSS_nrm = new int [lv_nra];

	#pragma endregion

	#pragma region "CREATE ACTIVITY PRIORITY SCORE STRUCTURE & BINARY ARRAY STRUCTURE (la_act_priority & la_bin_fin)"

	la_act_priority = new int [lv_nra];
	la_bin_fin = new int [lv_nrb];

	#pragma endregion

	#pragma region "CREATE PREDECESSOR AND RANKED SUCCESSOR STRUCTURE (lm_bin_tpre, lm_pre, lm_suc_ranked, lm_tpre, la_nrp, & la_nrtp)"

	lm_bin_tpre = new int * [lv_nra];
	lm_pre = new int * [lv_nra];
	lm_suc_ranked = new int * [lv_nra];
	lm_tpre = new int * [lv_nra];
	la_nrp = new int [lv_nra];
	la_nrtp = new int [lv_nra];
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		lm_bin_tpre[lv_j] = new int [lv_nrb];
		lm_pre[lv_j] = new int [lv_nra];
		lm_suc_ranked[lv_j] = new int [lv_nra];
		lm_tpre[lv_j] = new int [lv_nra];
	}

	#pragma endregion

	#pragma region "CREATE STATESPACE STRUCTURES (lm_state_rxa, lm_state_bin_fin, lm_state_bin_idl, lm_state_bus, lm_state_eli, lm_state_eli_act, lm_state_makespan, lm_state_prob, lm_state_tot_rate, lm_state_nrb, lm_state_nre, & la_state_nrz)"

	lm_state_rxa = new double ** [2];
	lm_state_bin_fin = new int ** [2];
	lm_state_bin_idl = new int ** [2];
	lm_state_bus = new int ** [2];
	lm_state_eli = new int ** [2];
	lm_state_eli_act = new bool ** [2];
	lm_state_makespan = new double * [2];
	lm_state_prob = new double * [2];
	lm_state_tot_rate = new double * [2];
	lm_state_nrb = new int * [2];
	lm_state_nre = new int * [2];
	for(lv_x=0;lv_x<2;lv_x++)
	{
		lm_state_rxa[lv_x] = new double * [lv_mxz];
		lm_state_bin_fin[lv_x] = new int * [lv_mxz];
		lm_state_bin_idl[lv_x] = new int * [lv_mxz];
		lm_state_bus[lv_x] = new int * [lv_mxz];
		lm_state_eli[lv_x] = new int * [lv_mxz];
		lm_state_eli_act[lv_x] = new bool * [lv_mxz];
		for(lv_z=0;lv_z<lv_mxz;lv_z++)
		{
			lm_state_rxa[lv_x][lv_z] = new double [lv_nrr];
			lm_state_bin_fin[lv_x][lv_z] = new int [lv_nrb];
			lm_state_bin_idl[lv_x][lv_z] = new int [lv_nrb];
			lm_state_bus[lv_x][lv_z] = new int [lv_mxp];
			lm_state_eli[lv_x][lv_z] = new int [lv_mxp];
			lm_state_eli_act[lv_x][lv_z] = new bool [lv_nra];
		}
		lm_state_makespan[lv_x] = new double [lv_mxz];
		lm_state_prob[lv_x] = new double [lv_mxz];
		lm_state_tot_rate[lv_x] = new double [lv_mxz];
		lm_state_nrb[lv_x] = new int [lv_mxz];
		lm_state_nre[lv_x] = new int [lv_mxz];
	}
	la_state_nrz = new int [2];

	#pragma endregion

	#pragma region "CREATE QUEUE STRUCTURE (la_queue)"

	la_queue = new int [(lv_nra*lv_nra)];

	#pragma endregion

	#pragma endregion

	#pragma region "[STEP 3b] EMPTY STRUCTURES REQUIRED IF THE PROJECT NETWORK DOES NOT HAVE A TOPOLOGICAL ORDER (INITIALIZED DURING EACH FUNCTION CALL)"

	#pragma region "DEFINE STRUCTURES"

	double ** lm_rxu_bck;	// [lv_j][lv_r]. Holds the use of resource lv_r by activity lv_j (backup).
	int ** lm_SS_pairs_bck;	// [lv_c][lv_k]. Holds the first activity (lv_k = 0) and the second activity (lv_k = 1) for SS pair lv_c (backup).
	int ** lm_suc_bck;	// [lv_j][lv_s]. Holds the lv_s^th successor of activity lv_j. lm_suc already contains additional FS precedence relations (backup).
	bool ** lm_suc_red;	// [lv_j][lv_s]. Holds whether or not the lv_s^th immediate successor of activity lv_j is redundant.

	double * la_dur_bck;	// [lv_j]. Holds the duration of activity lv_j (backup).
	double * la_rate_bck;	// [lv_j]. Holds the exponential rate (1 divided by the expected duration) of activity lv_j (backup).
	int * la_act2pos;	// [lv_j]. Holds the ordered (topological) position of the activity that has true ID lv_j.
	int * la_nrs_bck;	// [lv_j]. Holds the number of successors of activity lv_j. la_nrs already contains additional FS precedence relations (backup).
	int * la_pos2act;	// [lv_j]. Holds the true activity ID of the lv_j^th activity in a network that has been topologically ordered.
	int * la_priority_bck;	// [lv_i]. Holds the priority list (ranked list of activities, highest priority comes first) (backup).

	int lv_backup;	// Holds a backup variable.
	bool lv_redundant_suc;	// Holds whether or not there are redundant immediate successors (i.e., whether an activity has an immediate successor that is also a transitive successor of one of its immediate successors). Immediate successors can become redundant due to adding FS constraints.
	bool lv_top_order;	// Holds whether or not the network has a topological order.

	#pragma endregion

	#pragma region "CREATE BACKUP STRUCTURES THAT ARE REQUIRED IF THE PROJECT NETWORK DOES NOT HAVE A TOPOLOGICAL ORDER (lm_rxu_bck, lm_SS_pairs_bck, lm_suc_bck, la_dur_bck, la_rate_bck, la_act2pos, la_nrs_bck, la_pos2act, & la_priority_bck)"

	lm_rxu_bck = new double * [lv_nra];
	lm_suc_bck = new int * [lv_nra];
	lm_suc_red = new bool * [lv_nra];
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		lm_rxu_bck[lv_j] = new double [lv_nrr];
		lm_suc_bck[lv_j] = new int [lv_nra];
		lm_suc_red[lv_j] = new bool [lv_nra];
	}
	lm_SS_pairs_bck = new int * [lv_mxp_SS];
	for(lv_c=0;lv_c<lv_mxp_SS;lv_c++)
	{
		lm_SS_pairs_bck[lv_c] = new int [2];
	}
	la_dur_bck = new double [lv_nra];
	la_rate_bck = new double [lv_nra];
	la_act2pos = new int [lv_nra];
	la_nrs_bck = new int [lv_nra];
	la_pos2act = new int [lv_nra];
	la_priority_bck = new int [lv_nra];
	
	#pragma endregion

	#pragma endregion

	#pragma region "[STEP 4] DETERMINE USER INPUT (POLICY)"

	// The user has to supply the function with the following structures:
	//
	//	- int ** lm_SS_pairs.	[lv_c][lv_k].	Holds the first activity (lv_k = 0) and the second activity (lv_k = 1) for SS pair lv_c.
	//	- int ** lm_suc.		[lv_j][lv_s].	Holds the lv_s^th successor of activity lv_j. lm_suc already contains additional FS precedence relations.
	//	- int * la_nrs.			[lv_j].			Holds the number of successors of activity lv_j. la_nrs already contains additional FS precedence relations.
	//	- int * la_priority.	[lv_i].			Holds the priority list (ranked list of activities, highest priority comes first).
	//	- int lv_nrp_SS.						Holds the number of SS pairs.

	// Note: the successor structures (lm_suc & la_nrs) already have to contain any additional FS precedence relations imposed by the policy before being passed to the function.

	#pragma region "DEFINE STRUCTURES"

	int ** lm_SS_pairs;	// [lv_c][lv_k]. Holds the first activity (lv_k = 0) and second activity (lv_k = 1) of SS pair lv_c.
	int ** lm_suc;	// [lv_j][lv_s]. Successor lv_s of activity lv_j (also has to contain additional FS precedence relations).

	int * la_nrs;	// [lv_j]. Number of successors for activity lv_j (also has to contain additional FS precedence relations).
	int * la_priority;	// [lv_i]. Holds the priority list (ranked list of activities, highest priority comes first).

	int lv_nrp_SS;	// Holds the number of SS pairs.

	#pragma endregion

	// The code below serves the purpose of randomly generating a policy => this code is not required if the user provides the input himself.

	#pragma region "GENERATE A RANDOM POLICY"

	#pragma region "[NOT REQUIRED IN FUNCTION] LOCALS REQUIRED FOR GENERATING A RANDOM PRIORITY LIST/SET OF SS PAIRS"

	int ** lm_SS_pre;	// [lv_j][lv_p]. Predecessor lv_p of activity lv_j.
	int ** lm_SS_tpre;	// [lv_j][lv_p]. Transitive predecessor lv_p of activity lv_j.

	int * la_SS_nrp;	// [lv_j]. Number of predecessors for activity lv_j.
	int * la_SS_nrtp;	// [lv_j]. Number of transitive predecessors for activity lv_j.
	int * la_SS_queue;	// [lv_i]. Holds queue element lv_i (used to determine transitive structures).

	double lv_rand_switch;	// Holds the random number used to switch activities in the priority list.
	double lv_rand_SS;	// Holds the random number that determines how many SS pairs are identified.
	double lv_rand;	// Holds a random number.

	#pragma endregion

	#pragma region "[NOT REQUIRED IN FUNCTION] GET USER INPUT"

	cout << "Choose a number in (0;1) in order to determine how many random switches take place in order to construct the random priority list (smaller = less switching):" << "\n";
	cin >> lv_rand_switch;
	cout << "Choose a number in (0;1) in order to determine how many SS pairs are generated (smaller = less pairs):" << "\n";
	cin >> lv_rand_SS;

	#pragma endregion

	#pragma region "[NOT REQUIRED IN FUNCTION] GENERATE RANDOM PRIORITY LIST"

	#pragma region "GET RANDOM PRIORITY LIST"

	la_priority = new int [lv_nra];
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		la_priority[lv_j]=lv_j;	// Initialize priority list.
	}
	do
	{
		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			lv_rand=(static_cast<double>(rand())/static_cast<double>(RAND_MAX));	// Get random number in [0,1].
			if(lv_rand>0.5)	// Switch activity.
			{
				lv_rand=(static_cast<double>(rand())/static_cast<double>(RAND_MAX+1));	// Get random number in [0,1).
				lv_pos=static_cast<int>(floor(lv_rand*static_cast<double>(lv_nra)));	// Get random position.
				lv_act=la_priority[lv_pos];	// Get activity at position lv_pos.
				la_priority[lv_pos]=la_priority[lv_j];	// Switch activity.
				la_priority[lv_j]=lv_act;	// Switch activity.
			}
		}
		lv_rand=(static_cast<double>(rand())/static_cast<double>(RAND_MAX));	// Get random number in [0,1].
	}while(lv_rand<lv_rand_switch);	// Keep switching activities until a small enough random number is drawn.

	#pragma endregion

	#pragma region "PRINT RANDOM PRIORITY LIST"

	cout << "PRIORITY LIST:" << "\n";
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		cout << " " << la_priority[lv_j];
	}
	cout << "\n";

	#pragma endregion

	#pragma endregion

	#pragma region "[NOT REQUIRED IN FUNCTION] GET RANDOM SS PAIRS"

	#pragma region "CREATE SS PAIR PREDECESSOR STRUCTURE"

	lm_SS_pre = new int * [lv_nra];
	lm_SS_tpre = new int * [lv_nra];
	la_SS_nrp = new int [lv_nra];
	la_SS_nrtp = new int [lv_nra];
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		lm_SS_pre[lv_j] = new int [lv_nra];
		lm_SS_tpre[lv_j] = new int [lv_nra];
	}

	#pragma endregion

	#pragma region "GET SS PAIR PREDECESSOR STRUCTURE"

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		la_SS_nrp[lv_j]=0;	// Initialize number of predecessors of activity lv_j.
	}
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		for(lv_s=0;lv_s<la_nrs_ini[lv_j];lv_s++)
		{
			lv_suc=lm_suc_ini[lv_j][lv_s];	// Get successor lv_s of activity lv_j.
			lm_SS_pre[lv_suc][la_SS_nrp[lv_suc]]=lv_j;	// Activity lv_j is a predecessor of activity lv_suc.
			la_SS_nrp[lv_suc]++;	// Increment the number of predecessors of activity lv_suc.
		}
	}
	
	#pragma endregion

	#pragma region "GET SS PAIR TRANSITIVE PREDECESSOR STRUCTURES"

	#pragma region "INITIALIZE STRUCTURES"

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		la_SS_nrtp[lv_j]=0;	// Intialize the number of transitive predecessors.
	}
	la_SS_queue = new int [lv_nra];

	#pragma endregion

	for(lv_j=0;lv_j<lv_nra;lv_j++)	// Check all activities lv_j.
	{

		#pragma region "STEP 0: INITIALIZE QUEUE"

		lv_pos=0;	// Initialize the position in the queue.
		lv_size=0;	// Initialize the size of the queue.

		#pragma endregion

		#pragma region "STEP 1: STORE ALL SUCCESSORS OF ACTIVITY lv_j IN QUEUE"

		for(lv_s=0;lv_s<la_nrs_ini[lv_j];lv_s++)	// Check all successors of activity lv_j; each successor has lv_j as a predecessor.
		{
			lv_suc=lm_suc_ini[lv_j][lv_s];	// Set the successor of lv_j.
			lv_bool=false;	// Indicate that the (transitive) successor has not yet been identified.
			for(lv_i=0;lv_i<lv_size;lv_i++)
			{
				if(la_SS_queue[lv_i]==lv_suc)
				{
					lv_bool=true;	// Indicate that the (transitive) successor has already been identified.
					lv_i=lv_size;	// Exit the loop.
				}
			}
			if(lv_bool==false)	// The (transitive) successor is new => record it.
			{
				la_SS_queue[lv_size]=lv_suc;	// The lv_size^th successor of lv_j has lv_j as a predecessor & as such is stored in the queue.
				lv_size++;	// Increment the size of the queue.
			}
		}

		#pragma endregion

		#pragma region "STEP 2: FOR EACH SUCCESSOR IN THE QUEUE: ADD ITS SUCCESSORS TO THE QUEUE (they are the transitive successors of activity lv_j)"

		while(lv_size>lv_pos)	// Do until the end of the queue is reached.
		{
			for(lv_s=0;lv_s<la_nrs_ini[la_SS_queue[lv_pos]];lv_s++)	// Add the successors of the next queue element to the queue (these successors also have lv_j as a predecessor).
			{
				lv_suc=lm_suc_ini[la_SS_queue[lv_pos]][lv_s];	// Set the (transitive) successor of lv_j.
				lv_bool=false;	// Indicate that the (transitive) successor has not yet been identified.
				for(lv_i=0;lv_i<lv_size;lv_i++)
				{
					if(la_SS_queue[lv_i]==lv_suc)
					{
						lv_bool=true;	// Indicate that the (transitive) successor has already been identified.
						lv_i=lv_size;	// Exit the loop.
					}
				}
				if(lv_bool==false)	// The (transitive) successor is new => record it.
				{
					la_SS_queue[lv_size]=lv_suc;	// The lv_size^th successor of la_SS_queue[lv_pos] has lv_j as a transitive predecessor & as such is stored in the queue.
					lv_size++;	// Increment the size of the queue.
				}
			}
			lv_pos++;	// Increment the position in the queue.
		}

		#pragma endregion

		#pragma region "STEP 3: FOR EACH SUCCESSOR IN THE QUEUE: INDICATE THAT ACTIVITY lv_j IS A TRANSITIVE PREDECESSOR"

		for(lv_i=0;lv_i<lv_size;lv_i++)	// For all elements in the queue, set lv_j as a (transitive) predecessor.
		{
			lm_SS_tpre[la_SS_queue[lv_i]][la_SS_nrtp[la_SS_queue[lv_i]]]=lv_j;	// Copy lv_j as the (transitive) predecessor of the lv_i^th element in the queue. 
			la_SS_nrtp[la_SS_queue[lv_i]]++;	// Increment the number of (transitive) predecessors for the lv_i^th element in the queue.
		}

		#pragma endregion

	}

	#pragma endregion

	#pragma region "GENERATE RANDOM SS PAIRS"

	// This code requires la_priority to be initialized.

	lm_SS_pairs = new int * [lv_mxp_SS];
	lv_nrp_SS=0;	// Initialize the number of SS pairs.
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		for(lv_j2=(lv_j+1);lv_j2<lv_nra;lv_j2++)	// Check all other activities.
		{
			lv_bool=true;	// Initialize that activity lv_j2 is not a transitive predecessor of activity lv_j.
			for(lv_p=0;lv_p<la_SS_nrtp[lv_j];lv_p++)	// Check all transitive predecessors of activity lv_j.
			{
				if(lv_j2==lm_SS_tpre[lv_j][lv_p])
				{
					lv_bool=false;	// Indicate that activity lv_j2 is a transitive predecessor of activity lv_j. They cannot be executed together.
					break;
				}
			}
			if(lv_bool==true)	// Activity lv_j2 is not a transitive predecessor of activity lv_j.
			{
				for(lv_p=0;lv_p<la_SS_nrtp[lv_j2];lv_p++)	// Check all transitive predecessors of activity lv_j2.
				{
					if(lv_j==lm_SS_tpre[lv_j2][lv_p])
					{
						lv_bool=false;	// Indicate that activity lv_j is a transitive predecessor of activity lv_j2. They cannot be executed together.
						break;
					}
				}
				if(lv_bool==true)	// Activities lv_j and lv_j2 can be executed together.
				{
					for(lv_j3=0;lv_j3<lv_nra;lv_j3++)
					{
						if(la_priority[lv_j3]==lv_j)
						{
							break;	// Exit the loop.
						}
						else if(la_priority[lv_j3]==lv_j2)
						{
							lv_bool=false;	// Activity lv_j2 has a higher priority than activity lv_j. Activity lv_j2 cannot be started after activity lv_j.
							break;	// Exit the loop.
						}
					}
					if(lv_bool==true)	// Activities lv_j has a higher priority than activity lv_j2 => activity lv_j2 can start after activity lv_j.
					{
						for(lv_c=0;lv_c<lv_nrp_SS;lv_c++)	// Check all existing pairs.
						{
							if(lm_SS_pairs[lv_c][0]==lv_j2)	// Activity lv_j2 is the first member of pair lv_c.
							{
								if(lm_SS_pairs[lv_c][1]==lv_j)	// Activity lv_j is already paired with activity lv_j2.
								{
									lv_bool=false;	// Indicate that the pair has already been added.
									break;	// Exit loop.
								}
							}
						}
						if(lv_bool==true)	// The pair has not yet been added.
						{
							lv_rand=(static_cast<double>(rand())/static_cast<double>(RAND_MAX));	// Get random number in [0,1].
							if(lv_rand<lv_rand_SS)	// Add pair if random number is sufficiently small.
							{
								for(lv_r=0;lv_r<lv_nrr;lv_r++)	// Check all resources.
								{
									lv_rxu=la_rxa[lv_r];	// Initialize resource availability of resource lv_r.
									for(lv_c=0;lv_c<lv_nrp_SS;lv_c++)	// Check all existing pairs.
									{
										if(lm_SS_pairs[lv_c][0]==lv_j)	// Activity lv_j is a already a member of pair lv_c.
										{
											lv_act=lm_SS_pairs[lv_c][1];	// Get the other activity of the pair.
											lv_rxu-=lm_rxu[lv_act][lv_r];	// Update resource availability.
										}
										if(lm_SS_pairs[lv_c][1]==lv_j)	// Activity lv_j is a already a member of pair lv_c.
										{
											lv_act=lm_SS_pairs[lv_c][0];	// Get the other activity of the pair.
											lv_rxu-=lm_rxu[lv_act][lv_r];	// Update resource availability.
										}
										if(lm_SS_pairs[lv_c][0]==lv_j2)	// Activity lv_j2 is a already a member of pair lv_c.
										{
											lv_act=lm_SS_pairs[lv_c][1];	// Get the other activity of the pair.
											lv_rxu-=lm_rxu[lv_act][lv_r];	// Update resource availability.
										}
										if(lm_SS_pairs[lv_c][1]==lv_j2)	// Activity lv_j2 is a already a member of pair lv_c.
										{
											lv_act=lm_SS_pairs[lv_c][0];	// Get the other activity of the pair.
											lv_rxu-=lm_rxu[lv_act][lv_r];	// Update resource availability.
										}
									}
									lv_rxu-=lm_rxu[lv_j][lv_r];	// Update resource availability.
									lv_rxu-=lm_rxu[lv_j2][lv_r];	// Update resource availability.
									if(lv_rxu<0)	// Excessive resource use => pair is infeasible.
									{
										lv_bool=false;	// Indicate that the pair is infeasible.
										break;	// Exit loop.
									}
								}
								if(lv_bool==true)
								{
									lm_SS_pairs[lv_nrp_SS] = new int [2];
									lm_SS_pairs[lv_nrp_SS][0]=lv_j;	// Record the first activity of SS pair lv_nrp_SS.
									lm_SS_pairs[lv_nrp_SS][1]=lv_j2;	// Record the second activity of SS pair lv_nrp_SS.
									lv_nrp_SS++;	// Increment the number of SS pairs.
									if(lv_nrp_SS==lv_nra)
									{
										lv_j2=lv_nra;	// Exit loop. Sufficient SS pairs have been found.
										lv_j=lv_nra;	// Exit loop. Sufficient SS pairs have been found.
									}
								}
							}
						}
					}
				}
			}
		}
	}

	#pragma endregion

	#pragma region "PRINT RANDOM SS PAIRS"

	cout << "RANDOM SS PAIRS:" << "\n";
	for(lv_c=0;lv_c<lv_nrp_SS;lv_c++)
	{
		cout << " (" << lm_SS_pairs[lv_c][0] << " - " << lm_SS_pairs[lv_c][1] << ")";
	}
	cout << "\n";

	#pragma endregion

	#pragma endregion

	#pragma region "[NOT REQUIRED IN FUNCTION] GET SUCCESSOR STRUCTURE"

	lm_suc = new int * [lv_nra];
	la_nrs = new int [lv_nra];
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		lm_suc[lv_j] = new int [lv_nra];
		la_nrs[lv_j]=la_nrs_ini[lv_j];	// Copy successor structure from project file (i.e., do not add additional FS precedence relations).
		for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
		{
			lm_suc[lv_j][lv_s]=lm_suc_ini[lv_j][lv_s];	// Copy successor structure from project file (i.e., do not add additional FS precedence relations).
		}
	}

	#pragma endregion

	#pragma region "[NOT REQUIRED IN FUNCTION] DESTROY STRUCTURES"

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		delete lm_SS_pre[lv_j];
		delete lm_SS_tpre[lv_j];
	}
	delete lm_SS_pre;
	delete lm_SS_tpre;
	delete la_SS_nrp;
	delete la_SS_nrtp;
	delete la_SS_queue;

	#pragma endregion

	#pragma endregion

	#pragma endregion

	#pragma region "[STEP 4b] EXAMPLE POLICY"

	// First policy with error (J30 instance number 2).

	// Override policy.
	// Priority list:
	// 0,      2,      3,      1,      9,      11,     21,     4,      8,      5,      16,     15,     14,     12,     23,     6,      17,     19,     10,     18,     7,      28,     13,     22,     25,     20,     24,     30,     27,     26,     29,     31,

	// 3 additional FS constraints:
	// (25,27) - (16,1) - (8,14) where (A,B) means FS const. from A to B.

	//lm_suc[25][la_nrs[25]]=27;
	//la_nrs[25]++;
	//lm_suc[16][la_nrs[16]]=1;
	//la_nrs[16]++;
	//lm_suc[8][la_nrs[8]]=14;
	//la_nrs[8]++;

	//la_priority[0]=0;
	//la_priority[1]=2;
	//la_priority[2]=3;
	//la_priority[3]=1;
	//la_priority[4]=9;
	//la_priority[5]=11;
	//la_priority[6]=21;
	//la_priority[7]=4;
	//la_priority[8]=8;
	//la_priority[9]=5;
	//la_priority[10]=16;
	//la_priority[11]=15;
	//la_priority[12]=14;
	//la_priority[13]=12;
	//la_priority[14]=23;
	//la_priority[15]=6;
	//la_priority[16]=17;
	//la_priority[17]=19;
	//la_priority[18]=10;
	//la_priority[19]=18;
	//la_priority[20]=7;
	//la_priority[21]=28;
	//la_priority[22]=13;
	//la_priority[23]=22;
	//la_priority[24]=25;
	//la_priority[25]=20;
	//la_priority[26]=24;
	//la_priority[27]=30;
	//la_priority[28]=27;
	//la_priority[29]=26;
	//la_priority[30]=29;
	//la_priority[31]=31;

	// Second policy with error (J30 instance number 1). Gives error with SS (12 -> 6) & extra FS (17 -> 5).

	// priority list:
	// 0, 2, 3, 7, 12, 9, 8, 1, 6, 17, 4, 11, 18, 13, 10, 14, 16, 28, 15, 26, 19, 25, 24, 21, 20, 5, 27, 30, 22, 23, 29, 31,

	// SS ARCS:
	// from: 12, to: 6
	// from: 15, to: 5

	// FS ARCS:
	// from: 17, to: 5

	//lv_nrp_SS=0;
	//lm_SS_pairs[lv_nrp_SS][0]=12;
	//lm_SS_pairs[lv_nrp_SS][1]=6;
	//lv_nrp_SS++;
	//lm_SS_pairs[lv_nrp_SS][0]=15;
	//lm_SS_pairs[lv_nrp_SS][1]=5;
	//lv_nrp_SS++;

	//lm_suc[17][la_nrs[17]]=5;
	//la_nrs[17]++;

	//la_priority[0]=0;
	//la_priority[1]=2;
	//la_priority[2]=3;
	//la_priority[3]=7;
	//la_priority[4]=12;
	//la_priority[5]=9;
	//la_priority[6]=8;
	//la_priority[7]=1;
	//la_priority[8]=6;
	//la_priority[9]=17;
	//la_priority[10]=4;
	//la_priority[11]=11;
	//la_priority[12]=18;
	//la_priority[13]=13;
	//la_priority[14]=10;
	//la_priority[15]=14;
	//la_priority[16]=16;
	//la_priority[17]=28;
	//la_priority[18]=15;
	//la_priority[19]=26;
	//la_priority[20]=19;
	//la_priority[21]=25;
	//la_priority[22]=24;
	//la_priority[23]=21;
	//la_priority[24]=20;
	//la_priority[25]=5;
	//la_priority[26]=27;
	//la_priority[27]=30;
	//la_priority[28]=22;
	//la_priority[29]=23;
	//la_priority[30]=29;
	//la_priority[31]=31;

	#pragma endregion

	#pragma region "[STEP 5] FUNCTION CALL"

	// The user has to supply the function with the following structures (initialized by the user before calling the function):
	//
	//	- double ** lm_rxu.		[lv_j][lv_r].	Holds the use of resource lv_r by activity lv_j.
	//	- int ** lm_SS_pairs.	[lv_c][lv_k].	Holds the first activity (lv_k = 0) and the second activity (lv_k = 1) for SS pair lv_c.
	//	- int ** lm_suc.		[lv_j][lv_s].	Holds the lv_s^th successor of activity lv_j. lm_suc already contains additional FS precedence relations.
	//	- double * la_dur.		[lv_j].			Holds the duration of activity lv_j.
	//	- double * la_rxa.		[lv_r].			Holds the resource availability of resource lv_r.
	//	- int * la_nrs.			[lv_j].			Holds the number of successors of activity lv_j. la_nrs already contains additional FS precedence relations.
	//	- int * la_priority.	[lv_i].			Holds the priority list (ranked list of activities, highest priority comes first).
	//	- int lv_bits.							Holds the maximum number of bin members.
	//	- int lv_mxp.							Holds the maximum number of activities that are allowed 
	//	- int lv_nra.							Holds the number of activities.
	//	- int lv_nrp_SS.						Holds the number of SS pairs.
	//	- int lv_nrr.							Holds the number of resources.

	// The function also has to be supplied with the following initialized structures (initialized before the first function call):
	//
	//	- int ** lm_bin_act.	[lv_i][lv_m].	Holds the lv_m^th activity in bin lv_i.
	//	- double * la_rate.		[lv_j].			Holds the exponential duration rate of activity lv_j.
	//	- int * la_bin.			[lv_b].			Holds 2^lv_b at position lv_b.
	//	- int * la_bin_act.		[lv_j].			Holds the bin of activity lv_j.
	//	- int * la_bin_nra.		[lv_i].			Holds the number of activities in bin lv_i.
	//	- int * la_bin_val_act.	[lv_j].			Holds the binary value that represents activity lv_j.

	// The function also has to be supplied with the following empty structures (initialized during each function call):
	//
	//	- double *** lm_state_rxa.		[lv_x][lv_z][lv_r].	Holds the availability of resource lv_r in state lv_z for pivot lv_x.
	//	- int *** lm_state_bin_fin.		[lv_x][lv_z][lv_b].	For pivot lv_x holds the binary representation of the finished activities in state lv_z.
	//	- int *** lm_state_bin_idl.		[lv_x][lv_z][lv_b].	For pivot lv_x holds the binary representation of the idle activities in state lv_z.
	//	- int *** lm_state_bus.			[lv_x][lv_z][lv_m].	For pivot lv_x holds the lv_m^th ongoing activity in state lv_z.
	//	- int *** lm_state_eli.			[lv_x][lv_z][lv_e].	For pivot lv_x holds the lv_e^th eligible activity in state lv_z. Eligble activities are ranked according to their position in the priority list.
	//	- bool *** lm_state_eli_act.	[lv_x][lv_z][lv_j].	Holds whether activity lv_j is eligible to start or not for pivot lv_x and state lv_z.
	//	- double ** lm_state_makespan.	[lv_x][lv_z].		Holds the makespan for state lv_z at pivot lv_x.
	//	- double ** lm_state_prob.		[lv_x][lv_z].		Holds the probability to visit state lv_z at pivot lv_x.
	//	- double ** lm_state_tot_rate.	[lv_x][lv_z].		Holds the total rate of the ongoing activities in state lv_z for pivot lv_x.
	//	- int ** lm_aSS_bin.			[lv_j][lv_b].		Holds the binary representation of all pair members that need to be busy/finished in order to be able to start activity lv_j.
	//	- int ** lm_aSS_mem.			[lv_j][lv_m].		Holds the lv_m^th SS pair member of activity lv_j (all pair members have to be completed/ongoing before activity lv_j can be started).
	//	- int ** lm_bin_tpre.			[lv_j][lv_b].		Holds the binary representation of all transitive predecessors of activity lv_j.
	//	- int ** lm_pre.				[lv_j][lv_p].		Predecessor lv_p of activity lv_j.
	//	- int ** lm_state_nrb.			[lv_x][lv_z].		For pivot lv_x holds the number of activities that are ongoing in state lv_z.
	//	- int ** lm_state_nre.			[lv_x][lv_z].		For pivot lv_s holds the number of eligible activities in state lv_z.
	//	- int ** lm_suc_ranked.			[lv_j][lv_s].		Holds the lv_s^th ranked successor of activity lv_j.
	//	- int ** lm_tpre.				[lv_j][lv_p].		Transitive predecessor lv_p of activity lv_j.
	//	- int * la_aSS_nrm.				[lv_j].				Holds the number of SS pair members of activity lv_j (all pair members have to be completed/ongoing before activity lv_j can be started).
	//	- int * la_act_priority.		[lv_j].				Holds the priority score of activity lv_j (high score = high priority).
	//	- int * la_bin_fin.				[lv_b].				Holds the binary code of all finished activities.
	//	- int * la_nrp.					[lv_j].				Number of predecessors for activity lv_j.
	//	- int * la_nrtp.				[lv_j].				Number of transitive predecessors for activity lv_j.
	//	- int * la_queue.				[lv_i].				Holds queue element lv_i (used to determine transitive structures).
	//	- int * la_state_nrz.			[lv_x].				For pivot lv_x holds the number of states.

	// If the project network does not have a topological order, the function also has to be supplied with the following empty structures (initialized during each function call)
	//	- double ** lm_rxu_bck.			[lv_j][lv_r].		Holds the use of resource lv_r by activity lv_j (backup).
	//	- int ** lm_SS_pairs_bck.		[lv_c][lv_k].		Holds the first activity (lv_k = 0) and the second activity (lv_k = 1) for SS pair lv_c (backup).
	//	- int ** lm_suc_bck.			[lv_j][lv_s].		Holds the lv_s^th successor of activity lv_j. lm_suc already contains additional FS precedence relations (backup).
	//	- bool ** lm_suc_red;			[lv_j][lv_s].		Holds whether or not the lv_s^th immediate successor of activity lv_j is redundant.
	//	- double * la_dur_bck.			[lv_j].				Holds the duration of activity lv_j (backup).
	//	- int * la_act2pos.				[lv_j].				Holds the ordered (topological) position of the activity that has true ID lv_j.
	//	- int * la_nrs_bck.				[lv_j].				Holds the number of successors of activity lv_j. la_nrs already contains additional FS precedence relations (backup).
	//	- int * la_pos2act.				[lv_j].				Holds the true activity ID of the lv_j^th activity in a network that has been topologically ordered.
	//	- int * la_priority_bck.		[lv_i].				Holds the priority list (ranked list of activities, highest priority comes first) (backup).

	#pragma region "INITIALIZATION"

	#pragma region "INITIALIZE PREDECESSOR STRUCTURE (la_nrp & la_nrp)"

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		la_nrp[lv_j]=0;	// Initialize number of predecessors of activity lv_j.
	}
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
		{
			lv_suc=lm_suc[lv_j][lv_s];	// Get successor lv_s of activity lv_j.
			lm_pre[lv_suc][la_nrp[lv_suc]]=lv_j;	// Activity lv_j is a predecessor of activity lv_suc.
			la_nrp[lv_suc]++;	// Increment the number of predecessors of activity lv_suc.
		}
	}
	
	#pragma endregion

	#pragma region "INITIALIZE TRANSITIVE PREDECESSOR STRUCTURES (lm_tpre, & la_nrtp) AND CHECK FOR REDUNDANT IMMEDIATE SUCCESSORS (lm_suc_red)"

	// When adding FS constraints, it is possible that an activity is a direct and a transitive successor of another activity. This is not allowed. If an activity has another activity as an immediate and as a transitive successor, it is removed from its immediate successor list.

	#pragma region "INITIALIZE STRUCTURES"

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)	// Check all successors of activity lv_j; each successor has lv_j as a predecessor.
		{
			lm_suc_red[lv_j][lv_s]=false;	// Initialize that the lv_s^th successor of activity lv_j is not redundant.
		}
		la_nrtp[lv_j]=0;	// Intialize the number of transitive predecessors.
	}
	lv_redundant_suc=false;	// Initialize that there are no redundant immediate successors.

	#pragma endregion

	for(lv_j=0;lv_j<lv_nra;lv_j++)	// Check all activities lv_j.
	{

		#pragma region "STEP 0: INITIALIZE QUEUE"

		lv_pos=0;	// Initialize the position in the queue.
		lv_size=0;	// Initialize the size of the queue.

		#pragma endregion

		#pragma region "STEP 1: STORE ALL SUCCESSORS OF ACTIVITY lv_j IN QUEUE"

		for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)	// Check all successors of activity lv_j; each successor has lv_j as a predecessor.
		{
			lv_suc=lm_suc[lv_j][lv_s];	// Set the successor of lv_j.
			lv_bool=false;	// Indicate that the (transitive) successor has not yet been identified.
			for(lv_i=0;lv_i<lv_size;lv_i++)
			{
				if(la_queue[lv_i]==lv_suc)
				{
					lv_bool=true;	// Indicate that the (transitive) successor has already been identified.
					lv_i=lv_size;	// Exit the loop.
				}
			}
			if(lv_bool==false)	// The (transitive) successor is new => record it.
			{
				la_queue[lv_size]=lv_suc;	// The lv_size^th successor of lv_j has lv_j as a predecessor & as such is stored in the queue.
				lv_size++;	// Increment the size of the queue.
			}
		}

		#pragma endregion

		#pragma region "STEP 2: FOR EACH SUCCESSOR IN THE QUEUE: ADD ITS SUCCESSORS TO THE QUEUE (they are the transitive successors of activity lv_j) & CHECK WHETHER THEY ARE REDUNDANT IMMEDIATE SUCCESSORS OF ACTIVITY lv_j"

		while(lv_size>lv_pos)	// Do until the end of the queue is reached.
		{
			for(lv_s=0;lv_s<la_nrs[la_queue[lv_pos]];lv_s++)	// Add the successors of the next queue element to the queue (these successors also have lv_j as a predecessor).
			{
				lv_suc=lm_suc[la_queue[lv_pos]][lv_s];	// Set the (transitive) successor of lv_j.

				lv_bool=false;	// Indicate that the (transitive) successor is not an immediate successor of lv_j (otherwise lv_j has immediate successors that are also transitive successors).
				{
					for(lv_s2=0;lv_s2<la_nrs[lv_j];lv_s2++)	// Check all successors of activity lv_j; each successor has lv_j as a predecessor.
					{
						lv_suc2=lm_suc[lv_j][lv_s2];	// Set the successor of lv_j.
						if(lv_suc==lv_suc2)	// Transitive successor (lv_suc) is also an immediate successor (lv_suc2).
						{
							lv_redundant_suc=true;	// Indicate that there are redundant immediate successors.
							lm_suc_red[lv_j][lv_s2]=true;	// Indicate that the lv_s2^th successor of activity lv_j is redundant.
							lv_bool=true;	// Indicate that the transitive successor has already been identified.
						}
					}
				}
				if(lv_bool==false)	// The transitive successor is not an immediate successor, however, it can still already have been identified.
				{
					for(lv_i=0;lv_i<lv_size;lv_i++)
					{
						if(la_queue[lv_i]==lv_suc)
						{
							lv_bool=true;	// Indicate that the (transitive) successor has already been identified.
							lv_i=lv_size;	// Exit the loop.
						}
					}
				}
				if(lv_bool==false)	// The (transitive) successor is new => record it.
				{
					la_queue[lv_size]=lv_suc;	// The lv_size^th successor of la_queue[lv_pos] has lv_j as a transitive predecessor & as such is stored in the queue.
					lv_size++;	// Increment the size of the queue.
				}
			}
			lv_pos++;	// Increment the position in the queue.
		}

		#pragma endregion

		#pragma region "STEP 3: FOR EACH SUCCESSOR IN THE QUEUE: INDICATE THAT ACTIVITY lv_j IS A TRANSITIVE PREDECESSOR"

		for(lv_i=0;lv_i<lv_size;lv_i++)	// For all elements in the queue, set lv_j as a (transitive) predecessor.
		{
			lm_tpre[la_queue[lv_i]][la_nrtp[la_queue[lv_i]]]=lv_j;	// Copy lv_j as the (transitive) predecessor of the lv_i^th element in the queue. 
			la_nrtp[la_queue[lv_i]]++;	// Increment the number of (transitive) predecessors for the lv_i^th element in the queue.
		}

		#pragma endregion

	}

	#pragma endregion

	#pragma region "CHECK TOPOLOGICAL ORDER"

	// When adding FS constraints, it is possible that the topological order of the network is lost. This is not allowed.

	lv_top_order=true;	// Initialize that the network has a topological order.
	for(lv_j=0;lv_j<lv_nra;lv_j++)	// Check all activities lv_j.
	{
		for(lv_p=0;lv_p<la_nrtp[lv_j];lv_p++)
		{
			lv_pre=lm_tpre[lv_j][lv_p];	// Get predecessor activity.
			if(lv_pre>lv_j)	// The network does not have a topological order.
			{
				lv_top_order=false;	// Indicate that the network does not have a topological order.
				break;
			}
		}
	}

	#pragma endregion

	#pragma region "TOPOLOGICALLY ORDER THE NETWORK AND/OR REMOVE REDUNDANT SUCCESSORS IF NECESSARY"

	if(lv_top_order==false)
	{

		#pragma region "CREATE A BACKUP"

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			for(lv_r=0;lv_r<lv_nrr;lv_r++)
			{
				lm_rxu_bck[lv_j][lv_r]=lm_rxu[lv_j][lv_r];	// Create backup.
			}
			for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
			{
				lm_suc_bck[lv_j][lv_s]=lm_suc[lv_j][lv_s];	// Create backup.
			}
			la_dur_bck[lv_j]=la_dur[lv_j];	// Create backup.
			la_rate_bck[lv_j]=la_rate[lv_j];	// Create backup.
			la_nrs_bck[lv_j]=la_nrs[lv_j];	// Create backup.
			la_priority_bck[lv_j]=la_priority[lv_j];	// Create backup.
		}
		for(lv_c=0;lv_c<lv_nrp_SS;lv_c++)
		{
			lm_SS_pairs_bck[lv_c][0]=lm_SS_pairs[lv_c][0];	// Create backup.
			lm_SS_pairs_bck[lv_c][1]=lm_SS_pairs[lv_c][1];	// Create backup.
		}

		#pragma endregion

		#pragma region "PERFORM BUBBLE SORT TO FIND THE CORRECT ORDER"

		// Bubble sort is probably the best sorting algorithm to be used here because the initial ordering of the activity is still there. It are only the additional FS constraints that can cause the project to lose it's topological order.

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			la_pos2act[lv_j]=lv_j;	// Initialize the true ID of the activity at ordered (topological) position lv_j.
		}
		lv_j2=lv_nra;	// Initialize the number of unsorted activities.
		do
		{
			lv_bool=false;	// Initialize that no swap has been made.
			for(lv_j=1;lv_j<lv_j2;lv_j++)	// Check all unsorted activities.
			{
				if(la_nrtp[lv_j-1]>la_nrtp[lv_j])	// The activity at position lv_j-1 has more transitive predecessors than the activity at position lv_j => they need to be swapped.
				{
					lv_backup=la_nrtp[lv_j];	// Create a backup.
					la_nrtp[lv_j]=la_nrtp[lv_j-1];	// Swap the first element.
					la_nrtp[lv_j-1]=lv_backup;	// Swap the second element.
					lv_backup=la_pos2act[lv_j];	// Create a backup.
					la_pos2act[lv_j]=la_pos2act[lv_j-1];	// Swap the first element.
					la_pos2act[lv_j-1]=lv_backup;	// Swap the second element.
					lv_bool=true;	// A swap has been made.
				}
			}
			lv_j2--;	// Decrement the number of unsorted activities (an additional activity has been sorted during this loop).
		}while(lv_bool==true);	// Do until no swaps have been made.

		#pragma endregion

		#pragma region "INITIALIZE STRUCTURES WITH THE CORRECT (TOPOLOGICAL) ORDER"

		// la_pos2act[lv_j] holds the true ID of the activity at ordered (topological) position lv_j.
		// la_act2pos[lv_j] holds the ordered (topological) position of the activity that has true ID lv_j.

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			lv_act=la_pos2act[lv_j];	// Get the true ID of the activity at ordered (topological) position lv_j.
			la_act2pos[lv_act]=lv_j;	// Set the ordered (topological) position of the activity that has true ID lv_act.
		}
		for(lv_j=0;lv_j<lv_nra;lv_j++)	// Check all positions.
		{
			lv_act=la_pos2act[lv_j];	// Get the true ID of the activity at ordered (topological) position lv_j.
			for(lv_r=0;lv_r<lv_nrr;lv_r++)
			{
				lm_rxu[lv_j][lv_r]=lm_rxu_bck[lv_act][lv_r];	// Swap activity information.
			}
			for(lv_s=0;lv_s<la_nrs_bck[lv_act];lv_s++)
			{
				lv_suc=lm_suc_bck[lv_act][lv_s];	// Get the correct successor (true activity ID).
				lv_pos=la_act2pos[lv_suc];	// Get the ordered position of the correct successor.
				lm_suc[lv_j][lv_s]=lv_pos;	// Swap activity information.
			}
			la_dur[lv_j]=la_dur_bck[lv_act];	// Swap activity information.
			la_rate[lv_j]=la_rate_bck[lv_act];	// Swap activity information.
			la_nrs[lv_j]=la_nrs_bck[lv_act];	// Swap activity information.
			la_priority[lv_j]=la_priority_bck[lv_act];	// Swap activity information.
		}
		for(lv_c=0;lv_c<lv_nrp_SS;lv_c++)
		{
			lv_act=lm_SS_pairs_bck[lv_c][0];	// Get the true ID of the first activity of SS pair lv_c.
			lv_pos=la_act2pos[lv_act];	// Get the ordered (topological) position of the activity that has true ID lv_act.
			lm_SS_pairs[lv_c][0]=lv_pos;	// Swap activity.
			lv_act=lm_SS_pairs_bck[lv_c][1];	// Get the true ID of the second activity of SS pair lv_c.
			lv_pos=la_act2pos[lv_act];	// Get the ordered (topological) position of the activity that has true ID lv_act.
			lm_SS_pairs[lv_c][1]=lv_pos;	// Swap activity.
		}

		#pragma endregion

		#pragma region "REMOVE REDUNDANT SUCCESSORS IF NECESSARY"

		if(lv_redundant_suc==true)	// There are redundant successors.
		{
			for(lv_j=0;lv_j<lv_nra;lv_j++)
			{
				lv_act=la_pos2act[lv_j];	// Get the true ID of the activity at ordered (topological) position lv_j.
				lv_s2=0;	// Set number of non-redundant successors.
				for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
				{
					lv_suc=lm_suc[lv_j][lv_s];	// Get the correct successor.
					if(lm_suc_red[lv_act][lv_s]==false)	// The successor is not redundant.
					{
						lm_suc[lv_j][lv_s2]=lm_suc[lv_j][lv_s];	// Update successor structure.
						lv_s2++;	// Increment the number of non-redundant successors.
					}
				}
				la_nrs[lv_j]=lv_s2;	// Set the number of non-redundant successors.
			}
		}

		#pragma endregion

		#pragma region "INITIALIZE PREDECESSOR STRUCTURE (la_nrp & la_nrp)"

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			la_nrp[lv_j]=0;	// Initialize number of predecessors of activity lv_j.
		}
		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
			{
				lv_suc=lm_suc[lv_j][lv_s];	// Get successor lv_s of activity lv_j.
				lm_pre[lv_suc][la_nrp[lv_suc]]=lv_j;	// Activity lv_j is a predecessor of activity lv_suc.
				la_nrp[lv_suc]++;	// Increment the number of predecessors of activity lv_suc.
			}
		}
	
		#pragma endregion

		#pragma region "INITIALIZE TRANSITIVE PREDECESSOR STRUCTURES (lm_tpre, & la_nrtp)"

		#pragma region "INITIALIZE STRUCTURES"

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			la_nrtp[lv_j]=0;	// Intialize the number of transitive predecessors.
		}

		#pragma endregion

		for(lv_j=0;lv_j<lv_nra;lv_j++)	// Check all activities lv_j.
		{

			#pragma region "STEP 0: INITIALIZE QUEUE"

			lv_pos=0;	// Initialize the position in the queue.
			lv_size=0;	// Initialize the size of the queue.

			#pragma endregion

			#pragma region "STEP 1: STORE ALL SUCCESSORS OF ACTIVITY lv_j IN QUEUE"

			for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)	// Check all successors of activity lv_j; each successor has lv_j as a predecessor.
			{
				lv_suc=lm_suc[lv_j][lv_s];	// Set the successor of lv_j.
				lv_bool=false;	// Indicate that the (transitive) successor has not yet been identified.
				for(lv_i=0;lv_i<lv_size;lv_i++)
				{
					if(la_queue[lv_i]==lv_suc)
					{
						lv_bool=true;	// Indicate that the (transitive) successor has already been identified.
						lv_i=lv_size;	// Exit the loop.
					}
				}
				if(lv_bool==false)	// The (transitive) successor is new => record it.
				{
					la_queue[lv_size]=lv_suc;	// The lv_size^th successor of lv_j has lv_j as a predecessor & as such is stored in the queue.
					lv_size++;	// Increment the size of the queue.
				}
			}

			#pragma endregion

			#pragma region "STEP 2: FOR EACH SUCCESSOR IN THE QUEUE: ADD ITS SUCCESSORS TO THE QUEUE (they are the transitive successors of activity lv_j)"

			while(lv_size>lv_pos)	// Do until the end of the queue is reached.
			{
				for(lv_s=0;lv_s<la_nrs[la_queue[lv_pos]];lv_s++)	// Add the successors of the next queue element to the queue (these successors also have lv_j as a predecessor).
				{
					lv_suc=lm_suc[la_queue[lv_pos]][lv_s];	// Set the (transitive) successor of lv_j.
					lv_bool=false;	// Indicate that the (transitive) successor has not yet been identified.
					for(lv_i=0;lv_i<lv_size;lv_i++)
					{
						if(la_queue[lv_i]==lv_suc)
						{
							lv_bool=true;	// Indicate that the (transitive) successor has already been identified.
							lv_i=lv_size;	// Exit the loop.
						}
					}
					if(lv_bool==false)	// The (transitive) successor is new => record it.
					{
						la_queue[lv_size]=lv_suc;	// The lv_size^th successor of la_queue[lv_pos] has lv_j as a transitive predecessor & as such is stored in the queue.
						lv_size++;	// Increment the size of the queue.
					}
				}
				lv_pos++;	// Increment the position in the queue.
			}

			#pragma endregion

			#pragma region "STEP 3: FOR EACH SUCCESSOR IN THE QUEUE: INDICATE THAT ACTIVITY lv_j IS A TRANSITIVE PREDECESSOR"

			for(lv_i=0;lv_i<lv_size;lv_i++)	// For all elements in the queue, set lv_j as a (transitive) predecessor.
			{
				lm_tpre[la_queue[lv_i]][la_nrtp[la_queue[lv_i]]]=lv_j;	// Copy lv_j as the (transitive) predecessor of the lv_i^th element in the queue. 
				la_nrtp[la_queue[lv_i]]++;	// Increment the number of (transitive) predecessors for the lv_i^th element in the queue.
			}

			#pragma endregion

		}

		#pragma endregion

	}
	else if(lv_redundant_suc==true)	// The topological order is ok, however, there are redundant successors.
	{

		#pragma region "CREATE A BACKUP"

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
			{
				lm_suc_bck[lv_j][lv_s]=lm_suc[lv_j][lv_s];	// Create backup.
			}
			la_nrs_bck[lv_j]=la_nrs[lv_j];	// Create backup.
		}

		#pragma endregion

		#pragma region "REMOVE REDUNDANT SUCCESSORS"

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			lv_s2=0;	// Set number of non-redundant successors.
			for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
			{
				lv_suc=lm_suc[lv_j][lv_s];	// Get the correct successor.
				if(lm_suc_red[lv_j][lv_s]==false)	// The successor is not redundant.
				{
					lm_suc[lv_j][lv_s2]=lm_suc[lv_j][lv_s];	// Update successor structure.
					lv_s2++;	// Increment the number of non-redundant successors.
				}
			}
			la_nrs[lv_j]=lv_s2;	// Set the number of non-redundant successors.
		}

		#pragma endregion

		#pragma region "INITIALIZE PREDECESSOR STRUCTURE (la_nrp & la_nrp)"

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			la_nrp[lv_j]=0;	// Initialize number of predecessors of activity lv_j.
		}
		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
			{
				lv_suc=lm_suc[lv_j][lv_s];	// Get successor lv_s of activity lv_j.
				lm_pre[lv_suc][la_nrp[lv_suc]]=lv_j;	// Activity lv_j is a predecessor of activity lv_suc.
				la_nrp[lv_suc]++;	// Increment the number of predecessors of activity lv_suc.
			}
		}
	
		#pragma endregion

		#pragma region "INITIALIZE TRANSITIVE PREDECESSOR STRUCTURES (lm_tpre, & la_nrtp)"

		#pragma region "INITIALIZE STRUCTURES"

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			la_nrtp[lv_j]=0;	// Intialize the number of transitive predecessors.
		}

		#pragma endregion

		for(lv_j=0;lv_j<lv_nra;lv_j++)	// Check all activities lv_j.
		{

			#pragma region "STEP 0: INITIALIZE QUEUE"

			lv_pos=0;	// Initialize the position in the queue.
			lv_size=0;	// Initialize the size of the queue.

			#pragma endregion

			#pragma region "STEP 1: STORE ALL SUCCESSORS OF ACTIVITY lv_j IN QUEUE"

			for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)	// Check all successors of activity lv_j; each successor has lv_j as a predecessor.
			{
				lv_suc=lm_suc[lv_j][lv_s];	// Set the successor of lv_j.
				lv_bool=false;	// Indicate that the (transitive) successor has not yet been identified.
				for(lv_i=0;lv_i<lv_size;lv_i++)
				{
					if(la_queue[lv_i]==lv_suc)
					{
						lv_bool=true;	// Indicate that the (transitive) successor has already been identified.
						lv_i=lv_size;	// Exit the loop.
					}
				}
				if(lv_bool==false)	// The (transitive) successor is new => record it.
				{
					la_queue[lv_size]=lv_suc;	// The lv_size^th successor of lv_j has lv_j as a predecessor & as such is stored in the queue.
					lv_size++;	// Increment the size of the queue.
				}
			}

			#pragma endregion

			#pragma region "STEP 2: FOR EACH SUCCESSOR IN THE QUEUE: ADD ITS SUCCESSORS TO THE QUEUE (they are the transitive successors of activity lv_j)"

			while(lv_size>lv_pos)	// Do until the end of the queue is reached.
			{
				for(lv_s=0;lv_s<la_nrs[la_queue[lv_pos]];lv_s++)	// Add the successors of the next queue element to the queue (these successors also have lv_j as a predecessor).
				{
					lv_suc=lm_suc[la_queue[lv_pos]][lv_s];	// Set the (transitive) successor of lv_j.
					lv_bool=false;	// Indicate that the (transitive) successor has not yet been identified.
					for(lv_i=0;lv_i<lv_size;lv_i++)
					{
						if(la_queue[lv_i]==lv_suc)
						{
							lv_bool=true;	// Indicate that the (transitive) successor has already been identified.
							lv_i=lv_size;	// Exit the loop.
						}
					}
					if(lv_bool==false)	// The (transitive) successor is new => record it.
					{
						la_queue[lv_size]=lv_suc;	// The lv_size^th successor of la_queue[lv_pos] has lv_j as a transitive predecessor & as such is stored in the queue.
						lv_size++;	// Increment the size of the queue.
					}
				}
				lv_pos++;	// Increment the position in the queue.
			}

			#pragma endregion

			#pragma region "STEP 3: FOR EACH SUCCESSOR IN THE QUEUE: INDICATE THAT ACTIVITY lv_j IS A TRANSITIVE PREDECESSOR"

			for(lv_i=0;lv_i<lv_size;lv_i++)	// For all elements in the queue, set lv_j as a (transitive) predecessor.
			{
				lm_tpre[la_queue[lv_i]][la_nrtp[la_queue[lv_i]]]=lv_j;	// Copy lv_j as the (transitive) predecessor of the lv_i^th element in the queue. 
				la_nrtp[la_queue[lv_i]]++;	// Increment the number of (transitive) predecessors for the lv_i^th element in the queue.
			}

			#pragma endregion

		}

		#pragma endregion

	}

	#pragma endregion

	#pragma region "INITIALIZE BINARY TRANSITIVE PREDECESSOR STRUCTURE (lm_bin_tpre)"

	for(lv_j=0;lv_j<lv_nra;lv_j++)	// Check all activities lv_j.
	{
		for(lv_b=0;lv_b<lv_nrb;lv_b++)
		{
			lm_bin_tpre[lv_j][lv_b]=0;	// Initialize bin.
		}
		for(lv_p=0;lv_p<la_nrtp[lv_j];lv_p++)
		{
			lv_pre=lm_tpre[lv_j][lv_p];	// Get predecessor activity.
			lm_bin_tpre[lv_j][la_bin_act[lv_pre]]+=la_bin_val_act[lv_pre];	// Add binary value of the transitive predecessor to the correct bin.
		}
	}

	#pragma endregion

	#pragma region "INITIALIZE SS PAIR STRUCTURES (lm_aSS_bin, lm_aSS_mem, & la_aSS_nrm)"

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		for(lv_b=0;lv_b<lv_nrb;lv_b++)
		{
			lm_aSS_bin[lv_j][lv_b]=0;	// Initialize lm_aSS_bin.
		}
	}
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		la_aSS_nrm[lv_j]=0;	// Initialize the number of pair members of activity lv_j.
	}
	for(lv_c=0;lv_c<lv_nrp_SS;lv_c++)
	{
		lv_act=lm_SS_pairs[lv_c][0];	// Get first activity.
		lv_act2=lm_SS_pairs[lv_c][1];	// Get second activity.
		lv_bool=true;	// Initialize that activity lv_act is not yet recorded as an SS pair member of activity lv_act2 (activity lv_act has to be ongoing/completed before we can start activity lv_act2).
		for(lv_m=0;lv_m<la_aSS_nrm[lv_act2];lv_m++)
		{
			if(lm_aSS_mem[lv_act2][lv_m]==lv_act)
			{
				lv_bool=false;	// Activity lv_act has already been recorded as an SS pair member of activity lv_act2.
				break;	// Exit loop.
			}
		}
		if(lv_bool==true)	// Activity lv_act has not yet been recorded as an SS pair member of activity lv_act2 (activity lv_act has to be ongoing/completed before we can start activity lv_act2).
		{
			lm_aSS_mem[lv_act2][la_aSS_nrm[lv_act2]]=lv_act;	// Record activity lv_act as a SS pair member of activity lv_act2 (activity lv_act has to be ongoing/completed before we can start activity lv_act2).
			la_aSS_nrm[lv_act2]++;	// Increment the number of SS pair members of activity lv_act2.
		}
		lv_bin=la_bin_act[lv_act];	// Get the bin of activity lv_act.
		lv_bin_val=la_bin_val_act[lv_act];	// Get the binary value of activity lv_act.
		lm_aSS_bin[lv_act2][lv_bin]=(lm_aSS_bin[lv_act2][lv_bin] | lv_bin_val);	// Bitwise add the binary value of activity lv_act; the activity that has to be ongoing/completed before we can start activity lv_act2.
	}

	#pragma endregion

	#pragma region "INITIALIZE ACTIVITY PRIORITY SCORE STRUCTURE (la_act_priority)"

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		la_act_priority[la_priority[lv_j]]=(lv_nra-lv_j);	// Set the priority score of activity la_priority[lv_j] (i.e., the activity at the lv_j^th position in the priority list has priority score (lv_nra - lv_j) => activities that come first in the priority list have a higher priority score).
	}

	#pragma endregion

	#pragma region "INITIALIZE RANKED SUCCESSOR STRUCTURE (lm_suc_ranked)"

	// This code requires la_act_priority to be initialized.

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
		{
			la_queue[lv_s]=0;	// Initialize score of the lv_s^th successor activity.
		}
		for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
		{
			lv_suc=lm_suc[lv_j][lv_s];	// Get the lv_s^th successor of activity lv_j.
			for(lv_s2=0;lv_s2<la_nrs[lv_j];lv_s2++)
			{
				lv_suc2=lm_suc[lv_j][lv_s2];	// Get the lv_s2^th successor of activity lv_j.
				if(lv_suc!=lv_suc2)
				{
					if(la_act_priority[lv_suc]<la_act_priority[lv_suc2])	// Activity lv_suc has a lower priority than activity lv_s2.
					{
						la_queue[lv_s]++;	// Increment score of successor activity lv_s.
					}
				}
			}
			lm_suc_ranked[lv_j][la_queue[lv_s]]=lv_suc;	// Successor lv_suc has ranked position la_queue[lv_s] in the list of successor of activity lv_j.
		}
	}

	#pragma endregion

	#pragma region "INITIALIZE STATESPACE STRUCTURES (lm_state_rxa, lm_state_bin_fin, lm_state_bin_idl, lm_state_bus, lm_state_eli, lm_state_eli_act, lm_state_makespan, lm_state_prob, lm_state_tot_rate, lm_state_nrb, lm_state_nre, & la_state_nrz)"

	#pragma region "INITIALIZATION"

	la_state_nrz[0]=1;	// Initialize the number of states at pivot (lv_x = 0).
	la_state_nrz[1]=0;	// Initialize the number of states at pivot (lv_x = 1).
	lv_x=0;	// Set the current pivot.
	lv_x2=1;	// Set the next pivot.
	lv_z=0;	// Set the current state.
	for(lv_r=0;lv_r<lv_nrr;lv_r++)
	{
		lm_state_rxa[lv_x][lv_z][lv_r]=la_rxa[lv_r];	// Initialize the resource availability of resource lv_r in state lv_z at pivot lv_x.
	}
	for(lv_b=0;lv_b<lv_nrb;lv_b++)
	{
		lm_state_bin_idl[lv_x][lv_z][lv_b]=0;	// Initialize lm_state_bin_idl.
		lm_state_bin_fin[lv_x][lv_z][lv_b]=0;	// Initialize lm_state_bin_fin.
	}
	lv_bin=la_bin_act[0];	// Get the bin of the dummy-start activity.
	lv_bin_val=la_bin_val_act[0];	// Get the binary value of the dummy-start activity.
	lm_state_bin_fin[lv_x][lv_z][lv_bin]=lv_bin_val;	// Indicate that the dummy-start activity has finished.
	for(lv_j=1;lv_j<lv_nra;lv_j++)
	{
		lv_bin=la_bin_act[lv_j];	// Get the bin of activity lv_j.
		lv_bin_val=la_bin_val_act[lv_j];	// Get the binary value of activity lv_j.
		lm_state_bin_idl[lv_x][lv_z][lv_bin]+=lv_bin_val;	// Indicate that activity lv_j is idle.
	}
	lm_state_nrb[lv_x][lv_z]=0;	// Indicate that no activity is currently ongoing.
	lm_state_nre[lv_x][lv_z]=0;	// Initialize the number of eligible activities.
	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		lm_state_eli_act[lv_x][lv_z][lv_j]=false;	// Initialize that activity lv_j is not eligible to start in state lv_z at pivot lv_x.
	}
	lm_state_makespan[lv_x][lv_z]=0.0;	// Initialize the makespan at the start of the project.
	lm_state_prob[lv_x][lv_z]=1.0;	// Initialize the probability to visit state lv_z at pivot lv_x.
	lm_state_tot_rate[lv_x][lv_z]=0.0;	// Initialize the total exponential rate of ongoing activities in state lv_z at pivot lv_x.

	#pragma endregion

	#pragma region "SET FIRST QUEUE ELEMENT"

	// In order to determine the first queue element, we:
	//	- Determine set of eligible activities (ranked according to their priority score).
	//	- Start as many activities as possible.
	//	- Update the set of eligible activities.

	#pragma region "DETERMINE SET OF RANKED ELIGIBLE ACTIVITIES (DUMMY-START)"

	// Note: we assume that successor activities are ranked according to their priority score.

	for(lv_s=0;lv_s<la_nrs[0];lv_s++)	// Check all successors of the dummy-start activity.
	{
		lv_suc=lm_suc_ranked[0][lv_s];	// Get the lv_s^th ranked successor of the dummy-start activity.
		lm_state_eli_act[lv_x][lv_z][lv_suc]=true;	// Indicate that activity lv_suc is eligible to start in state lv_z at pivot lv_x.
		lm_state_eli[lv_x][lv_z][lv_s]=lv_suc;	// Record activity lv_suc (i.e., the lv_s^th successor of the dummy-start activity) as the lv_s^th eligible activity.
		lm_state_nre[lv_x][lv_z]++;	// Increment the number of eligible activities.
	}

	#pragma endregion

	#pragma region "START ACTIVITIES"

	for(lv_e=0;lv_e<lm_state_nre[lv_x][lv_z];lv_e++)	// Check all eligible activities in state lv_z at pivot lv_x.
	{
		lv_eli=lm_state_eli[lv_x][lv_z][lv_e];	// Get the eligible activity.
		if(la_aSS_nrm[lv_eli]>0)	// Activity lv_eli has SS pair members that have to be ongoing/completed before it can be started.
		{
			lv_bool=true;	// Initialize that all SS pair members are ongoing/have been completed.
			for(lv_b=0;lv_b<lv_nrb;lv_b++)
			{
				if((lm_state_bin_idl[lv_x][lv_z][lv_b] & lm_aSS_bin[lv_eli][lv_b])>0)	// At least one SS pair member is idle (i.e., is not ongoing/completed) => activity lv_eli cannot be started.
				{
					lv_bool=false;	// Indicate that activity lv_eli cannot be started because not all of its SS pair members are ongoing/have been completed.
					break;	// Exit the loop.
				}
			}
			if(lv_bool==true)	// All SS pair members are ongoing/have been completed.
			{
				for(lv_r=0;lv_r<lv_nrr;lv_r++)
				{
					lm_state_rxa[lv_x][lv_z][lv_r]-=lm_rxu[lv_eli][lv_r];	// Capture resource lv_r.
					if(lm_state_rxa[lv_x][lv_z][lv_r]<0)	// Insufficient resources are available to start activity lv_eli.
					{
						do
						{
							lm_state_rxa[lv_x][lv_z][lv_r]+=lm_rxu[lv_eli][lv_r];	// Reset resource availability.
							lv_r--;
						}while(lv_r>=0);	// Do until all resource availabilities have been reset.
						lv_bool=false;	// Indicate that activity lv_eli cannot be started.
						break;	// Exit the loop.
					}

				}
			}
		}
		else	// Activity lv_eli does not have any SS pair members => it can be started regardless of the status of other activities.
		{
			lv_bool=true;	// Initialize that activity lv_eli can be started.
			for(lv_r=0;lv_r<lv_nrr;lv_r++)
			{
				lm_state_rxa[lv_x][lv_z][lv_r]-=lm_rxu[lv_eli][lv_r];	// Capture resource lv_r.
				if(lm_state_rxa[lv_x][lv_z][lv_r]<0)	// Insufficient resources are available to start activity lv_eli.
				{
					do
					{
						lm_state_rxa[lv_x][lv_z][lv_r]+=lm_rxu[lv_eli][lv_r];	// Reset resource availability.
						lv_r--;
					}while(lv_r>=0);	// Do until all resource availabilities have been reset.
					lv_bool=false;	// Indicate that activity lv_eli cannot be started.
					break;	// Exit the loop.
				}
			}
		}
		if(lv_bool==true)	// Activity lv_eli has been started.
		{
			lm_state_eli_act[lv_x][lv_z][lv_eli]=false;	// Indicate that activity lv_eli is no longer eligible to start.
			lm_state_bus[lv_x][lv_z][lm_state_nrb[lv_x][lv_z]]=lv_eli;	// Record activity lv_eli as the lm_state_nrb[lv_x][lv_z]^th busy activity.
			lm_state_nrb[lv_x][lv_z]++;	// Increment the number of busy activities.
			lm_state_tot_rate[lv_x][lv_z]+=la_rate[lv_eli];	// Update total exponential rate of the busy activities.
			lv_bin=la_bin_act[lv_eli];	// Get the bin of activity lv_eli.
			lv_bin_val=la_bin_val_act[lv_eli];	// Get the binary value of activity lv_eli.
			lm_state_bin_idl[lv_x][lv_z][lv_bin]-=lv_bin_val;	// Activity lv_eli is no longer idle.
		}
	}

	#pragma endregion

	#pragma region "ADAPT SET OF RANKED ELIGIBLE ACTIVITIES"

	lv_e2=0;	// Initialize eligible activity position.
	for(lv_e=0;lv_e<lm_state_nre[lv_x][lv_z];lv_e++)	// Check all eligible activities in state lv_z at pivot lv_x.
	{
		lv_eli=lm_state_eli[lv_x][lv_z][lv_e];	// Get the eligible activity.
		if(lm_state_eli_act[lv_x][lv_z][lv_eli]==true)	// Activity lv_eli is still eligible.
		{
			lm_state_eli[lv_x][lv_z][lv_e2]=lv_eli;	// Record activity lv_eli as the lv_e2^th eligible activity.
			lv_e2++;	// Increment eligible activity position.
		}
	}
	lm_state_nre[lv_x][lv_z]=lv_e2;	// Set the number of eligible activities.

	#pragma endregion

	#pragma endregion

	#pragma endregion

	#pragma endregion

	#pragma region "PROCESS QUEUE"

	lv_act_done=0;	// Initialize that 0 activities have been processed.
	do
	{
		lv_dur=(1.0/lm_state_tot_rate[lv_x][lv_z]);	// Get the expected time spent in the current state.
		for(lv_n=0;lv_n<lm_state_nrb[lv_x][lv_z];lv_n++)	// Check all busy activities in state lv_z at pivot lv_x.
		{

			#pragma region "INITIALIZE BASIC STATE INFORMATION"

			lv_act=lm_state_bus[lv_x][lv_z][lv_n];	// Get the lv_n^th busy activity.
			lv_prob=(la_rate[lv_act]/lm_state_tot_rate[lv_x][lv_z]);	// Get the probability that activity lv_act finished first.
			lv_bin=la_bin_act[lv_act];	// Get the bin of activity lv_act.
			lv_bin_val=la_bin_val_act[lv_act];	// Get the binary value of activity lv_act.
			for(lv_b=0;lv_b<lv_nrb;lv_b++)
			{
				la_bin_fin[lv_b]=lm_state_bin_fin[lv_x][lv_z][lv_b];	// Initialize bin of finished activities.
			}
			la_bin_fin[lv_bin]+=lv_bin_val;	// Activity lv_act completes => add its binary value to the correct bin (i.e., bin lv_bin).

			#pragma endregion

			#pragma region "DETERMINE IF STATE IS NEW & UPDATE EXISTING STATE IF IT IS NOT"

			lv_new=true;	// Initialize that the current state is new.
			if(la_state_nrz[lv_x2]>0)	// States have been recorded at pivot lv_x2 => do a search to see whether the new state already exists/to see where to insert the current state.
			{
				for(lv_z2=0;lv_z2<la_state_nrz[lv_x2];lv_z2++)
				{
					lv_bool=true;	// Initialize that the current state is state lv_z2.
					for(lv_b=0;lv_b<lv_nrb;lv_b++)
					{
						if(lm_state_bin_fin[lv_x2][lv_z2][lv_b]!=la_bin_fin[lv_b])
						{
							lv_bool=false;	// Indicate that the current state is not state lv_z2.
							break;	// Exit the loop.
						}
					}
					if(lv_bool==true)	// The current state is state lv_z2.
					{
						lv_prob2=(lm_state_prob[lv_x][lv_z]*lv_prob);	// Get probability to visit state lv_z2 at pivot lv_x2 when departing from state lv_z at pivot lv_x.
						lv_prob3=lm_state_prob[lv_x2][lv_z2];	// Get probability to visit state lv_z2 at pivot lv_x2 when departing from another state at pivor lv_x.
						lv_prob4=(lv_prob2+lv_prob3);	// Get total probability to visit state lv_z2 at pivot lv_x2.
						lv_prob2=(lv_prob2/lv_prob4);	// Get weight applied to makespan when departing from state lv_z at pivot lv_x.
						lv_prob3=(lv_prob3/lv_prob4);	// Get weight applied to makespan when departing from another state at pivot lv_x.
						lv_dur2=(lm_state_makespan[lv_x][lv_z]+lv_dur);	// Get time to visit state lv_z2 at pivot lv_x when departing from state lv_z at pivot lv_x.
						lm_state_makespan[lv_x2][lv_z2]=((lv_dur2*lv_prob2)+(lm_state_makespan[lv_x2][lv_z2]*lv_prob3));	// Set makespan upon entering state lv_z2 at pivot lv_x2.
						lm_state_prob[lv_x2][lv_z2]=lv_prob4;	// Set the probability to visit state lv_z2 at pivot lv_x2.
						lv_new=false;	// Indicate that the current state is not new.
						break;	// Exit the loop.
					}
				}
			}

			#pragma endregion

			#pragma region "ADD NEW STATE"

			if(lv_new==true)	// The state is new.
			{
				lv_z2=la_state_nrz[lv_x2];	// Set state.

				#pragma region "INITIALIZE STRUCTURES OF THE NEW STATE"

				for(lv_r=0;lv_r<lv_nrr;lv_r++)
				{
					lm_state_rxa[lv_x2][lv_z2][lv_r]=(lm_state_rxa[lv_x][lv_z][lv_r]+lm_rxu[lv_act][lv_r]);	// Set the resource availability of resource lv_r.
				}
				for(lv_b=0;lv_b<lv_nrb;lv_b++)
				{
					lm_state_bin_fin[lv_x2][lv_z2][lv_b]=la_bin_fin[lv_b];	// Set binary code that represents the finished activities.
					lm_state_bin_idl[lv_x2][lv_z2][lv_b]=lm_state_bin_idl[lv_x][lv_z][lv_b];	// Create a copy of lm_state_bin_idl.
				}
				lv_n2=0;	// Initialize busy activity counter.
				lv_n3=0;	// Initialize future busy activity counter.
				do
				{
					if(lv_n!=lv_n2)	// The busy activity is not the ongoing activity that finishes.
					{
						lm_state_bus[lv_x2][lv_z2][lv_n3]=lm_state_bus[lv_x][lv_z][lv_n2];	// Create a copy of the lv_n2^th busy activity.
						lv_n3++;	// Increment future busy activity counter.
					}
					lv_n2++;	// Increment busy activity counter.
				}while(lv_n2<lm_state_nrb[lv_x][lv_z]);	// Do until all busy activities have been processed.
				lm_state_nrb[lv_x2][lv_z2]=lv_n3;	// Set the number of ongoing activities.
				for(lv_j=0;lv_j<lv_nra;lv_j++)
				{
					lm_state_eli_act[lv_x2][lv_z2][lv_j]=lm_state_eli_act[lv_x][lv_z][lv_j];	// Create a copy of the boolean structure that indicates whether or not an activity is eligible to start.
				}
				lm_state_makespan[lv_x2][lv_z2]=(lm_state_makespan[lv_x][lv_z]+lv_dur);	// Set the makespan upon entering state lv_z2 at pivot lv_x2.
				lm_state_prob[lv_x2][lv_z2]=(lm_state_prob[lv_x][lv_z]*lv_prob);	// Set the probability to visit state lv_z2 at pivot lv_x2.
				lm_state_tot_rate[lv_x2][lv_z2]=(lm_state_tot_rate[lv_x][lv_z]-la_rate[lv_act]);	// Set the total exponential rate of ongoing activities in state lv_z2 at pivot lv_x2.

				#pragma endregion

				#pragma region "DETERMINE SET OF RANKED ELIGIBLE ACTIVITIES (I.E., INITIALIZE lm_state_eli_act & lm_state_nre)"

				// In order to determine the set of ranked eligible activities we perform two steps:
				//	- Obtain set of new eligible activities from the set of ranked successor activities.
				//	- Combine the set of new & existing eligible actities.

				lv_nre_new=0;	// Initialize the number of new eligible activities.
				for(lv_s=0;lv_s<la_nrs[lv_act];lv_s++)	// Check all successors of the activity that completes.
				{
					lv_suc=lm_suc_ranked[lv_act][lv_s];	// Get the successor activity.
					if(lm_state_eli_act[lv_x2][lv_z2][lv_suc]==false)	// The successor has not yet been made eligible.
					{
						lv_bool=true;	// Initialize that the successor activity becomes eligible (i.e., that all of its transitive predecessors have finished).
						for(lv_b=0;lv_b<lv_nrb;lv_b++)
						{
							if((lm_bin_tpre[lv_suc][lv_b] & la_bin_fin[lv_b]) != lm_bin_tpre[lv_suc][lv_b])	// Not all transitive predecessors have finished.
							{
								lv_bool=false;	// Indicate that the successor activity is not eligible yet.
								break;	// Exit the loop.
							}
						}
						if(lv_bool==true)	// The successor activity is eligible.
						{
							lm_state_eli_act[lv_x2][lv_z2][lv_suc]=true;	// Indicate that activity lv_suc is eligible to start in state lv_z2 at pivot lv_x2.
							la_queue[lv_nre_new]=lv_suc;	// Record new eligible activity.
							lv_nre_new++;	// Increment the number of new eligible activities.
						}
					}
				}
				lm_state_nre[lv_x2][lv_z2]=0;	// Initialize the number of eligible activities (new & those that were already identified). Note that it is possible that there are no eligible activities. It is impossible, however, to have no eligible activities nor ongoing activities.
				if(lv_nre_new==0)	// No new eligble activities have been identified. All eligible activities were already identified.
				{
					for(lv_e=0;lv_e<lm_state_nre[lv_x][lv_z];lv_e++)
					{
						lm_state_eli[lv_x2][lv_z2][lv_e]=lm_state_eli[lv_x][lv_z][lv_e];	// Copy existing eligible activity.
						lm_state_nre[lv_x2][lv_z2]++;	// Increment the number of eligible activities.
					}
				}
				else if(lm_state_nre[lv_x][lv_z]==0)	// There are no existing eligible activities. All eligible activities are new.
				{
					// Note: we assume that successor activities are ranked according to their priority score.
					for(lv_e2=0;lv_e2<lv_nre_new;lv_e2++)
					{
						lm_state_eli[lv_x2][lv_z2][lv_e2]=la_queue[lv_e2];	// Copy new eligible activity.
						lm_state_nre[lv_x2][lv_z2]++;	// Increment the number of eligible activities.
					}
				}
				else	// There are both new and already identified eligible activities.
				{
					lv_e=0;	// Initialize counter of existing eligible activities.
					lv_e2=0;	// Initialize counter of new eligible activities.
					lv_eli=lm_state_eli[lv_x][lv_z][lv_e];	// Get existing eligible activity.
					lv_priority=la_act_priority[lv_eli];	// Get priority score of the first existing eligible activity.
					lv_eli_new=la_queue[lv_e2];	// Get new eligible activity.
					lv_priority_new=la_act_priority[lv_eli_new];	// Get priority score of the first new eligible activity.
					do
					{
						if(lv_priority>lv_priority_new)	// Activity lv_eli (the existing eligible activity) has a higher priority.
						{
							lm_state_eli[lv_x2][lv_z2][lm_state_nre[lv_x2][lv_z2]]=lv_eli;	// Record existing eligible activity lv_eli as the lm_state_nre[lv_x2][lv_z2]^th eligible activity.
							lv_e++;	// Increment the number of existing eligible activities.
							if(lv_e==lm_state_nre[lv_x][lv_z])	// No more existing eligible activities exist.
							{
								lv_priority=-1;	// Set priority of existing eligible activities.
							}
							else
							{
								lv_eli=lm_state_eli[lv_x][lv_z][lv_e];	// Get existing eligible activity.
								lv_priority=la_act_priority[lv_eli];	// Get priority score of the lv_e^th existing eligible activity.
							}
						}
						else	// Activity lv_eli_new (the new eligible activity) has a higher priority.
						{
							lm_state_eli[lv_x2][lv_z2][lm_state_nre[lv_x2][lv_z2]]=lv_eli_new;	// Record new eligible activity lv_eli_new as the lm_state_nre[lv_x2][lv_z2]^th eligible activity.
							lv_e2++;	// Increment the number of new eligible activities.
							if(lv_e2==lv_nre_new)	// No more new eligible activities exist.
							{
								lv_priority_new=-1;	// Set priority of new eligible activities.
							}
							else
							{
								lv_eli_new=la_queue[lv_e2];	// Get new eligible activity.
								lv_priority_new=la_act_priority[lv_eli_new];	// Get priority score of the lv_e2^th new eligible activity.
							}
						}
						lm_state_nre[lv_x2][lv_z2]++;	// Increment the number of eligible activities.
					}while((lv_priority>-1) || (lv_priority_new>-1));	// Do until all eligible activities have been ranked.
				}

				#pragma endregion

				#pragma region "START ACTIVITIES"

				for(lv_e=0;lv_e<lm_state_nre[lv_x2][lv_z2];lv_e++)	// Check all eligible activities.
				{
					lv_eli=lm_state_eli[lv_x2][lv_z2][lv_e];	// Get the eligible activity.
					if(la_aSS_nrm[lv_eli]>0)	// Activity lv_eli has SS pair members that have to be ongoing/completed before it can be started.
					{
						lv_bool=true;	// Initialize that all SS pair members are ongoing/have been completed.
						for(lv_b=0;lv_b<lv_nrb;lv_b++)
						{
							if((lm_state_bin_idl[lv_x2][lv_z2][lv_b] & lm_aSS_bin[lv_eli][lv_b])>0)	// At least one SS pair member is idle (i.e., is not ongoing/completed) => activity lv_eli cannot be started.
							{
								lv_bool=false;	// Indicate that activity lv_eli cannot be started because not all of its SS pair members are ongoing/have been completed.
								break;	// Exit the loop.
							}
						}
						if(lv_bool==true)	// All SS pair members are ongoing/have been completed.
						{
							for(lv_r=0;lv_r<lv_nrr;lv_r++)
							{
								lm_state_rxa[lv_x2][lv_z2][lv_r]-=lm_rxu[lv_eli][lv_r];	// Capture resource lv_r.
								if(lm_state_rxa[lv_x2][lv_z2][lv_r]<0)	// Insufficient resources are available to start activity lv_eli.
								{
									do
									{
										lm_state_rxa[lv_x2][lv_z2][lv_r]+=lm_rxu[lv_eli][lv_r];	// Reset resource availability.
										lv_r--;
									}while(lv_r>=0);	// Do until all resource availabilities have been reset.
									lv_bool=false;	// Indicate that activity lv_eli cannot be started.
									break;	// Exit the loop.
								}
							}
						}
					}
					else	// Activity lv_eli does not have any SS pair members => it can be started regardless of the status of other activities.
					{
						lv_bool=true;	// Initialize that activity lv_eli can be started.
						for(lv_r=0;lv_r<lv_nrr;lv_r++)
						{
							lm_state_rxa[lv_x2][lv_z2][lv_r]-=lm_rxu[lv_eli][lv_r];	// Capture resource lv_r.
							if(lm_state_rxa[lv_x2][lv_z2][lv_r]<0)	// Insufficient resources are available to start activity lv_eli.
							{
								do
								{
									lm_state_rxa[lv_x2][lv_z2][lv_r]+=lm_rxu[lv_eli][lv_r];	// Reset resource availability.
									lv_r--;
								}while(lv_r>=0);	// Do until all resource availabilities have been reset.
								lv_bool=false;	// Indicate that activity lv_eli cannot be started.
								break;	// Exit the loop.
							}
						}
					}
					if(lv_bool==true)	// Activity lv_eli has been started.
					{
						lm_state_eli_act[lv_x2][lv_z2][lv_eli]=false;	// Indicate that activity lv_eli is no longer eligible to start.
						lm_state_bus[lv_x2][lv_z2][lm_state_nrb[lv_x2][lv_z2]]=lv_eli;	// Record activity lv_eli as the lm_state_nrb[lv_x2][lv_z2]^th busy activity.
						lm_state_nrb[lv_x2][lv_z2]++;	// Increment the number of busy activities.
						lm_state_tot_rate[lv_x2][lv_z2]+=la_rate[lv_eli];	// Update total exponential rate of the busy activities.
						lv_bin=la_bin_act[lv_eli];	// Get the bin of activity lv_eli.
						lv_bin_val=la_bin_val_act[lv_eli];	// Get the binary value of activity lv_eli.
						lm_state_bin_idl[lv_x2][lv_z2][lv_bin]-=lv_bin_val;	// Activity lv_eli is no longer idle.
					}
				}

				#pragma endregion

				#pragma region "ADAPT SET OF RANKED ELIGIBLE ACTIVITIES"

				lv_e2=0;	// Initialize eligible activity position.
				for(lv_e=0;lv_e<lm_state_nre[lv_x2][lv_z2];lv_e++)	// Check all eligible activities in state lv_z2 at pivot lv_x2.
				{
					lv_eli=lm_state_eli[lv_x2][lv_z2][lv_e];	// Get the eligible activity.
					if(lm_state_eli_act[lv_x2][lv_z2][lv_eli]==true)	// Activity lv_eli is still eligible.
					{
						lm_state_eli[lv_x2][lv_z2][lv_e2]=lv_eli;	// Record activity lv_eli as the lv_e2^th eligible activity.
						lv_e2++;	// Increment eligible activity position.
					}
				}
				lm_state_nre[lv_x2][lv_z2]=lv_e2;	// Set the number of eligible activities.

				#pragma endregion

				la_state_nrz[lv_x2]++;	// Increment the number of states at pivot lv_x2.
				if(la_state_nrz[lv_x2]>=lv_mxz)
				{
					int lv_error;
					cout << "\n";
					cout << "Number of states (" << la_state_nrz[lv_x2] << ") exceeds maximum number of states (" << lv_mxz << ").";
					cout << "\n";
					cout << "Press any key to quit...";
					cin >> lv_error;
					throw 0;
				}
			}

			#pragma endregion

		}
		lv_z++;	// Increment the current state.
		if(lv_z==la_state_nrz[lv_x])	// All states have been processed at the current pivot.
		{
			//double lv_pmass;
			//lv_pmass=0;
			//for(lv_z2=0;lv_z2<la_state_nrz[lv_x2];lv_z2++)
			//{
			//	lv_pmass+=lm_state_prob[lv_x2][lv_z2];
			//}
			//double lv_makespan;
			//lv_makespan=0;
			//for(lv_z2=0;lv_z2<la_state_nrz[lv_x2];lv_z2++)
			//{
			//	lv_makespan+=(lm_state_prob[lv_x2][lv_z2]*lm_state_makespan[lv_x2][lv_z2]);
			//}
			la_state_nrz[lv_x]=0;	// Reset the number of states.
			lv_z=0;	// Reset current state.
			lv_x2=lv_x;	// Switch pivot.
			lv_x=(1-lv_x);	// Switch pivot.
			lv_act_done++;	// Increment the number of activities that have been processed.
			cout << "[" << lv_act_done << "]";
		}
	}while(la_state_nrz[lv_x]>0);	// Do until there are no more remaining states (i.e., the dummy-end activity has finished).
	cout << "\n";
	for(lv_r=0;lv_r<lv_nrr;lv_r++)
	{
		cout << "\n" << "RX (" << lv_r << ") AVAILABILITY (" << la_rxa[lv_r] << ") REMAIN (" << lm_state_rxa[lv_x][0][lv_r] << ")";
	}
	cout << "\n";
	cout << "\n" << "PROBABILITY MASS : " << lm_state_prob[lv_x][0];
	cout << "\n" << "MAKESPAN         : " << lm_state_makespan[lv_x][0];
	cout << "\n" << "WEIGHTED MAKESPAN: " << (lm_state_makespan[lv_x][0]/lm_state_prob[lv_x][0]);

	#pragma endregion

	#pragma region "SIMULATION"

	// The simulation serves the purpose of error-checking the function.
	// The simulation code can be deleted/is not required for the function.

	#pragma region "DEFINE SIMULATION LOCALS"

	double * la_sim_act_fin;	// [lv_j]. Holds the finish time for each activity.
	double * la_sim_dur;	// [lv_j]. Holds a random variate of the duration of activity lv_j.
	double * la_sim_rxa;	// [lv_r]. Holds the resource availability of resource lv_r.
	int * la_sim_bin_fin;	// [lv_b]. Holds the binary code that represents the finished activities.
	int * la_sim_bin_idl;	// [lv_b]. Holds the binary code that represents the idle activities.
	int * la_sim_bus;	// [lv_m]. Holds the lv_m^th busy activity.
	int * la_sim_eli;	// [lv_e]. Holds the lv_e^th eligible activity.
	int * la_sim_eli2;	// [lv_e2]. Holds the lv_e2^th new eligible activity.
	bool * la_sim_bool_eli;	// [lv_j]. Holds whether activity lv_j is eligible or not.
	
	double lv_sim_cur_time;	// Holds the current time.
	double lv_sim_makespan;	// Holds the expected makespan.
	double lv_sim_nxt_time;	// Holds the time of the next event.
	int lv_sim_nrb;	// Holds the number of busy activities.
	int lv_sim_nrb2;	// Holds the number of new busy activities.
	int lv_sim_nre;	// Holds the number of eligible activities.
	int lv_sim_nre2;	// Holds the number of new & existing eligible activities.
	int lv_sim_nri;	// Holds the number of simulation iterations.
	int lv_sim_nxt_act;	// Holds the activity that completes next.

	#pragma endregion

	#pragma region "CREATE SIMULATION STRUCTURES"

	la_sim_act_fin = new double [lv_nra];
	la_sim_dur = new double [lv_nra];
	la_sim_rxa = new double [lv_nrr];
	la_sim_bin_fin = new int [lv_nrb];
	la_sim_bin_idl = new int [lv_nrb];
	la_sim_bus = new int [lv_nra];
	la_sim_eli = new int [lv_nra];
	la_sim_eli2 = new int [lv_nra];
	la_sim_bool_eli = new bool [lv_nra];

	#pragma endregion

	#pragma region "SET NUMBER OF SIMULATION ITERATIONS (lv_sim_nri)"
	
	lv_sim_nri=1000000;	// Set the number of simulation iterations.

	#pragma endregion

	#pragma region "PERFORM SIMULATION"

	lv_sim_makespan=0.0;	// Initialize the expected makespan.
	for(lv_i=0;lv_i<lv_sim_nri;lv_i++)
	{

		#pragma region "INITIALIZE SIMULATION ITERATION lv_i"

		lv_sim_cur_time=0.0;	// Initialize the current time.
		la_sim_dur[0]=0;	// Set duration of the dummy-start activity.
		la_sim_bool_eli[0]=false;	// Indicate that the dummy-start activity is not eligible.
		la_sim_dur[(lv_nra-1)]=0;	// Set duration of the dummy-end activity.
		la_sim_bool_eli[(lv_nra-1)]=false;	// Indicate that the dummy-end activity is not eligible.
		for(lv_b=0;lv_b<lv_nrb;lv_b++)
		{
			la_sim_bin_fin[lv_b]=0;	// Initialize bin of finished activities.
			la_sim_bin_idl[lv_b]=0;	// Initialize bin of idle activities.
		}
		lv_bin=la_bin_act[0];	// Get the bin of the dummy-start activity.
		lv_bin_val=la_bin_val_act[0];	// Get the binary value of the dummy-start activity.
		la_sim_bin_fin[lv_bin]+=lv_bin_val;	// Set bin of finished activity (the dummy-start activity is finished).
		lv_sim_nxt_time=1.0;	// Initialize next event time.
		for(lv_j=1;lv_j<(lv_nra-1);lv_j++)	// Check all non-dummy activities.
		{
			la_sim_dur[lv_j]=-(log(1.0-(static_cast<double>(rand()))/(RAND_MAX+1))/la_rate[lv_j]);	// Get random variate of the exponential distribution.
			la_sim_bool_eli[lv_j]=false;	// Indicate that activity lv_j is not eligible.
			lv_sim_nxt_time+=la_sim_dur[lv_j];	// Initialize next event time.
			lv_bin=la_bin_act[lv_j];	// Get the bin of activity lv_j.
			lv_bin_val=la_bin_val_act[lv_j];	// Get the binary value of activity lv_j.
			la_sim_bin_idl[lv_bin]+=lv_bin_val;	// Set bin of idle activities.
		}
		lv_bin=la_bin_act[(lv_nra-1)];	// Get the bin of the dummy-end activity.
		lv_bin_val=la_bin_val_act[(lv_nra-1)];	// Get the binary value of the dummy-end activity.
		la_sim_bin_idl[lv_bin]+=lv_bin_val;	// Set bin of idle activities.
		for(lv_r=0;lv_r<lv_nrr;lv_r++)
		{
			la_sim_rxa[lv_r]=la_rxa[lv_r];	// Initialize the availability of resource lv_r.
		}
		lv_sim_nxt_act=0;	// Initialize the next activity that finishes.
		lv_sim_nrb=0;	// Initialize the number of busy activities.

		#pragma endregion

		#pragma region "INITIALIZE QUEUE"

		#pragma region "DETERMINE SET OF RANKED ELIGIBLE ACTIVITIES (DUMMY-START)"

		lv_sim_nre=0;	// Initialize the number of eligible activities.
		for(lv_s=0;lv_s<la_nrs[0];lv_s++)	// Check all successors of the dummy-start activity.
		{
			lv_suc=lm_suc_ranked[0][lv_s];	// Get the lv_s^th ranked successor of the dummy-start activity.
			la_sim_eli[lv_sim_nre]=lv_suc;	// Indicate that activity lv_suc is the lv_sim_nre^th eligible activity.
			la_sim_bool_eli[lv_suc]=true;	// Indicate that activity lv_suc is eligible.
			lv_sim_nre++;	// Increment the number of eligible activities.
		}

		#pragma endregion

		#pragma region "START ACTIVITIES"

		for(lv_e=0;lv_e<lv_sim_nre;lv_e++)	// Check all eligible activities.
		{
			lv_eli=la_sim_eli[lv_e];	// Get the eligible activity.
			if(la_aSS_nrm[lv_eli]>0)	// Activity lv_eli has SS pair members that have to be ongoing/completed before it can be started.
			{
				lv_bool=true;	// Initialize that all SS pair members are ongoing/have been completed.
				for(lv_b=0;lv_b<lv_nrb;lv_b++)
				{
					if((la_sim_bin_idl[lv_b] & lm_aSS_bin[lv_eli][lv_b])>0)	// At least one SS pair member is idle (i.e., is not ongoing/completed) => activity lv_eli cannot be started.
					{
						lv_bool=false;	// Indicate that activity lv_eli cannot be started because not all of its SS pair members are ongoing/have been completed.
						break;	// Exit the loop.
					}
				}
				if(lv_bool==true)	// All SS pair members are ongoing/have been completed.
				{
					for(lv_r=0;lv_r<lv_nrr;lv_r++)
					{
						la_sim_rxa[lv_r]-=lm_rxu[lv_eli][lv_r];	// Capture resource lv_r.
						if(la_sim_rxa[lv_r]<0)	// Insufficient resources are available to start activity lv_eli.
						{
							do
							{
								la_sim_rxa[lv_r]+=lm_rxu[lv_eli][lv_r];	// Reset resource availability.
								lv_r--;
							}while(lv_r>=0);	// Do until all resource availabilities have been reset.
							lv_bool=false;	// Indicate that activity lv_eli cannot be started.
							break;	// Exit the loop.
						}

					}
				}
			}
			else	// Activity lv_eli does not have any SS pair members => it can be started regardless of the status of other activities.
			{
				lv_bool=true;	// Initialize that activity lv_eli can be started.
				for(lv_r=0;lv_r<lv_nrr;lv_r++)
				{
					la_sim_rxa[lv_r]-=lm_rxu[lv_eli][lv_r];	// Capture resource lv_r.
					if(la_sim_rxa[lv_r]<0)	// Insufficient resources are available to start activity lv_eli.
					{
						do
						{
							la_sim_rxa[lv_r]+=lm_rxu[lv_eli][lv_r];	// Reset resource availability.
							lv_r--;
						}while(lv_r>=0);	// Do until all resource availabilities have been reset.
						lv_bool=false;	// Indicate that activity lv_eli cannot be started.
						break;	// Exit the loop.
					}
				}
			}
			if(lv_bool==true)	// Activity lv_eli has been started.
			{
				la_sim_act_fin[lv_eli]=(lv_sim_cur_time+la_sim_dur[lv_eli]);	// Set the finish time of activity lv_eli.
				lv_bin=la_bin_act[lv_eli];	// Get the bin of activity lv_eli.
				lv_bin_val=la_bin_val_act[lv_eli];	// Get the binary value of activity lv_eli.
				la_sim_bin_idl[lv_bin]-=lv_bin_val;	// Activity lv_eli is no longer idle.
				la_sim_bool_eli[lv_eli]=false;	// Indicate that activity lv_eli is no longer eligible.
				la_sim_bus[lv_sim_nrb]=lv_eli;	// Record activity lv_eli as the lv_sim_nrb^th busy activity.
				lv_sim_nrb++;	// Increment the number of busy activities.
			}
		}

		#pragma endregion

		#pragma region "SET NEXT EVENT TIME"

		lv_act=la_sim_bus[0];	// Get the first busy activity.
		lv_sim_nxt_time=la_sim_act_fin[lv_act];	// Initialize the next event time.
		lv_sim_nxt_act=lv_act;	// Initialize activity that triggers the next event.
		for(lv_m=1;lv_m<lv_sim_nrb;lv_m++)
		{
			lv_act=la_sim_bus[lv_m];	// Get the lv_m^th busy activity.
			if(la_sim_act_fin[lv_act]<lv_sim_nxt_time)
			{
				lv_sim_nxt_time=la_sim_act_fin[lv_act];	// Record new next event time.
				lv_sim_nxt_act=lv_act;	// Record activity that triggers the next event.
			}
		}

		#pragma endregion

		#pragma region "ADAPT SET OF RANKED ELIGIBLE ACTIVITIES"

		lv_e2=0;	// Initialize eligible activity position.
		for(lv_e=0;lv_e<lv_sim_nre;lv_e++)	// Check all eligible activities.
		{
			lv_eli=la_sim_eli[lv_e];	// Get the eligible activity.
			if(la_sim_bool_eli[lv_eli]==true)	// Activity lv_eli is still eligible.
			{
				la_sim_eli[lv_e2]=lv_eli;	// Record activity lv_eli as the lv_e2^th eligible activity.
				lv_e2++;	// Increment eligible activity position.
			}
		}
		lv_sim_nre=lv_e2;	// Set the number of eligible activities.

		#pragma endregion

		#pragma endregion

		#pragma region "PROCESS QUEUE"

		do
		{

			#pragma region "EXECUTE NEXT EVENT/COMPLETE NEXT ACTIVITY & UPDATE CURRENT TIME"

			lv_act=lv_sim_nxt_act;	// Get activity that finishes next.
			lv_bin=la_bin_act[lv_act];	// Get the bin of activity lv_act.
			lv_bin_val=la_bin_val_act[lv_act];	// Get the binary value of activity lv_act.
			la_sim_bin_fin[lv_bin]+=lv_bin_val;	// Update binary code that represents the finished activities.
			lv_sim_cur_time=lv_sim_nxt_time;	// The next event takes place => update the current time.
			for(lv_r=0;lv_r<lv_nrr;lv_r++)
			{
				la_sim_rxa[lv_r]+=lm_rxu[lv_act][lv_r];	// Release resource lv_r used by the activity that finishes next.
			}
			lv_sim_nrb2=0;	// Initialize the number of new busy activities.
			for(lv_m=0;lv_m<lv_sim_nrb;lv_m++)
			{
				if(la_sim_bus[lv_m]!=lv_act)
				{
					la_sim_bus[lv_sim_nrb2]=la_sim_bus[lv_m];
					lv_sim_nrb2++;	// Increment the number of new busy activities.
				}
			}
			lv_sim_nrb--;	// Decrement the number of busy activities.
			//cout << "[" << lv_act << "]";

			#pragma endregion

			#pragma region "DETERMINE SET OF RANKED ELIGIBLE ACTIVITIES"

			lv_nre_new=0;	// Initialize the number of new eligible activities.
			for(lv_s=0;lv_s<la_nrs[lv_act];lv_s++)	// Check all successors of the activity that completes.
			{
				lv_suc=lm_suc_ranked[lv_act][lv_s];	// Get the successor activity.
				if(la_sim_bool_eli[lv_suc]==false)	// The successor has not yet been made available.
				{
					lv_bool=true;	// Initialize that the successor activity becomes eligible (i.e., that all of its transitive predecessors have finished).
					for(lv_b=0;lv_b<lv_nrb;lv_b++)
					{
						if((lm_bin_tpre[lv_suc][lv_b] & la_sim_bin_fin[lv_b]) != lm_bin_tpre[lv_suc][lv_b])	// Not all transitive predecessors have finished.
						{
							lv_bool=false;	// Indicate that the successor activity is not eligible yet.
							break;	// Exit the loop.
						}
					}
					if(lv_bool==true)	// The successor activity is eligible.
					{
						la_sim_bool_eli[lv_suc]=true;	// Indicate that activity lv_suc is eligible to start.
						la_queue[lv_nre_new]=lv_suc;	// Record new eligible activity.
						lv_nre_new++;	// Increment the number of new eligible activities.
					}
				}
			}
			lv_sim_nre2=0;	// Initialize the number of new & existing eligible activities.
			if(lv_nre_new==0)	// No new eligble activities have been identified. All eligible activities were already identified.
			{
				for(lv_e=0;lv_e<lv_sim_nre;lv_e++)
				{
					la_sim_eli[lv_e]=la_sim_eli[lv_e];	// Copy existing eligible activity.
					lv_sim_nre2++;	// Increment the number of new & existing eligible activities.
				}
			}
			else if(lv_sim_nre==0)	// There are no existing eligible activities. All eligible activities are new.
			{
				// Note: we assume that successor activities are ranked according to their priority score.
				for(lv_e2=0;lv_e2<lv_nre_new;lv_e2++)
				{
					la_sim_eli[lv_e2]=la_queue[lv_e2];	// Copy new eligible activity.
					lv_sim_nre2++;	// Increment the number of new & existing eligible activities.
				}
			}
			else	// There are both new and already identified eligible activities.
			{
				lv_e=0;	// Initialize counter of existing eligible activities.
				lv_e2=0;	// Initialize counter of new eligible activities.
				lv_sim_nre2=0;	// Initialize counter of new & existing eligible activities.
				lv_eli=la_sim_eli[lv_e];	// Get existing eligible activity.
				lv_priority=la_act_priority[lv_eli];	// Get priority score of the first existing eligible activity.
				lv_eli_new=la_queue[lv_e2];	// Get new eligible activity.
				lv_priority_new=la_act_priority[lv_eli_new];	// Get priority score of the first new eligible activity.
				do
				{
					if(lv_priority>lv_priority_new)	// Activity lv_eli (the existing eligible activity) has a higher priority.
					{
						la_sim_eli2[lv_sim_nre2]=lv_eli;	// Record existing eligible activity lv_eli as the lv_sim_nre2^th eligible activity.
						lv_e++;	// Increment the number of existing eligible activities.
						if(lv_e==lv_sim_nre)	// No more existing eligible activities exist.
						{
							lv_priority=-1;	// Set priority of existing eligible activities.
						}
						else
						{
							lv_eli=la_sim_eli[lv_e];	// Get existing eligible activity.
							lv_priority=la_act_priority[lv_eli];	// Get priority score of the lv_e^th existing eligible activity.
						}
					}
					else	// Activity lv_eli_new (the new eligible activity) has a higher priority.
					{
						la_sim_eli2[lv_sim_nre2]=lv_eli_new;	// Record new eligible activity lv_eli_new as the lv_sim_nre2^th eligible activity.
						lv_e2++;	// Increment the number of new eligible activities.
						if(lv_e2==lv_nre_new)	// No more new eligible activities exist.
						{
							lv_priority_new=-1;	// Set priority of new eligible activities.
						}
						else
						{
							lv_eli_new=la_queue[lv_e2];	// Get new eligible activity.
							lv_priority_new=la_act_priority[lv_eli_new];	// Get priority score of the lv_e2^th new eligible activity.
						}
					}
					lv_sim_nre2++;	// Increment the number of new & existing eligible activities.
				}while((lv_priority>-1) || (lv_priority_new>-1));	// Do until all eligible activities have been ranked.
				for(lv_e2=0;lv_e2<lv_sim_nre2;lv_e2++)
				{
					la_sim_eli[lv_e2]=la_sim_eli2[lv_e2];	// Set the ranked eligible activites.
				}
			}
			lv_sim_nre=lv_sim_nre2;	// Set the number of new & existing eligible activities.

			#pragma endregion

			#pragma region "START ACTIVITIES"

			for(lv_e=0;lv_e<lv_sim_nre;lv_e++)	// Check all eligible activities.
			{
				lv_eli=la_sim_eli[lv_e];	// Get the eligible activity.
				if(la_aSS_nrm[lv_eli]>0)	// Activity lv_eli has SS pair members that have to be ongoing/completed before it can be started.
				{
					lv_bool=true;	// Initialize that all SS pair members are ongoing/have been completed.
					for(lv_b=0;lv_b<lv_nrb;lv_b++)
					{
						if((la_sim_bin_idl[lv_b] & lm_aSS_bin[lv_eli][lv_b])>0)	// At least one SS pair member is idle (i.e., is not ongoing/completed) => activity lv_eli cannot be started.
						{
							lv_bool=false;	// Indicate that activity lv_eli cannot be started because not all of its SS pair members are ongoing/have been completed.
							break;	// Exit the loop.
						}
					}
					if(lv_bool==true)	// All SS pair members are ongoing/have been completed.
					{
						for(lv_r=0;lv_r<lv_nrr;lv_r++)
						{
							la_sim_rxa[lv_r]-=lm_rxu[lv_eli][lv_r];	// Capture resource lv_r.
							if(la_sim_rxa[lv_r]<0)	// Insufficient resources are available to start activity lv_eli.
							{
								do
								{
									la_sim_rxa[lv_r]+=lm_rxu[lv_eli][lv_r];	// Reset resource availability.
									lv_r--;
								}while(lv_r>=0);	// Do until all resource availabilities have been reset.
								lv_bool=false;	// Indicate that activity lv_eli cannot be started.
								break;	// Exit the loop.
							}

						}
					}
				}
				else	// Activity lv_eli does not have any SS pair members => it can be started regardless of the status of other activities.
				{
					lv_bool=true;	// Initialize that activity lv_eli can be started.
					for(lv_r=0;lv_r<lv_nrr;lv_r++)
					{
						la_sim_rxa[lv_r]-=lm_rxu[lv_eli][lv_r];	// Capture resource lv_r.
						if(la_sim_rxa[lv_r]<0)	// Insufficient resources are available to start activity lv_eli.
						{
							do
							{
								la_sim_rxa[lv_r]+=lm_rxu[lv_eli][lv_r];	// Reset resource availability.
								lv_r--;
							}while(lv_r>=0);	// Do until all resource availabilities have been reset.
							lv_bool=false;	// Indicate that activity lv_eli cannot be started.
							break;	// Exit the loop.
						}
					}
				}
				if(lv_bool==true)	// Activity lv_eli has been started.
				{
					la_sim_act_fin[lv_eli]=(lv_sim_cur_time+la_sim_dur[lv_eli]);	// Set the finish time of activity lv_eli.
					lv_bin=la_bin_act[lv_eli];	// Get the bin of activity lv_eli.
					lv_bin_val=la_bin_val_act[lv_eli];	// Get the binary value of activity lv_eli.
					la_sim_bin_idl[lv_bin]-=lv_bin_val;	// Activity lv_eli is no longer idle.
					la_sim_bool_eli[lv_eli]=false;	// Indicate that activity lv_eli is no longer eligible.
					la_sim_bus[lv_sim_nrb]=lv_eli;	// Record activity lv_eli as the lv_sim_nrb^th busy activity.
					lv_sim_nrb++;	// Increment the number of busy activities.
				}
			}

			#pragma endregion

			#pragma region "SET NEXT EVENT TIME"

			lv_act2=la_sim_bus[0];	// Get the first busy activity.
			lv_sim_nxt_time=la_sim_act_fin[lv_act2];	// Initialize the next event time.
			lv_sim_nxt_act=lv_act2;	// Initialize activity that triggers the next event.
			for(lv_m=1;lv_m<lv_sim_nrb;lv_m++)
			{
				lv_act2=la_sim_bus[lv_m];	// Get the lv_m^th busy activity.
				if(la_sim_act_fin[lv_act2]<lv_sim_nxt_time)
				{
					lv_sim_nxt_time=la_sim_act_fin[lv_act2];	// Record new next event time.
					lv_sim_nxt_act=lv_act2;	// Record activity that triggers the next event.
				}
			}

			#pragma endregion

			#pragma region "ADAPT SET OF RANKED ELIGIBLE ACTIVITIES"

			lv_e2=0;	// Initialize eligible activity position.
			for(lv_e=0;lv_e<lv_sim_nre;lv_e++)	// Check all eligible activities.
			{
				lv_eli=la_sim_eli[lv_e];	// Get the eligible activity.
				if(la_sim_bool_eli[lv_eli]==true)	// Activity lv_eli is still eligible.
				{
					la_sim_eli[lv_e2]=lv_eli;	// Record activity lv_eli as the lv_e2^th eligible activity.
					lv_e2++;	// Increment eligible activity position.
				}
			}
			lv_sim_nre=lv_e2;	// Set the number of eligible activities.

			#pragma endregion

		}while(lv_sim_nxt_act!=(lv_nra-1));	// Do until the dummy-end activity is the next activity that finishes.

		#pragma endregion

		lv_sim_makespan+=la_sim_act_fin[(lv_nra-1)];	// Get total makespan.
	}
	lv_sim_makespan=(lv_sim_makespan/static_cast<double>(lv_sim_nri));	// Get the expected makespan.
	cout << "\n";
	cout << "\n" << "ITERATIONS (SIM) : " << lv_sim_nri;
	cout << "\n" << "MAKESPAN (SIM)   : " << lv_sim_makespan;

	#pragma endregion

	#pragma region "DESTROY SIMULATION STRUCTURES"

	delete la_sim_act_fin;
	delete la_sim_dur;
	delete la_sim_rxa;
	delete la_sim_bin_fin;
	delete la_sim_bin_idl;
	delete la_sim_bus;
	delete la_sim_eli;
	delete la_sim_eli2;
	delete la_sim_bool_eli;

	#pragma endregion

	#pragma endregion

	#pragma region "RESTORE BACKUP IF THE NETWORK DID NOT HAVE A TOPOLOGICAL ORDER"

	if(lv_top_order==false)
	{

		#pragma region "RESTORE BACKUP"

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			for(lv_r=0;lv_r<lv_nrr;lv_r++)
			{
				lm_rxu[lv_j][lv_r]=lm_rxu_bck[lv_j][lv_r];	// Create backup.
			}
			for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
			{
				lm_suc[lv_j][lv_s]=lm_suc_bck[lv_j][lv_s];	// Create backup.
			}
			la_dur[lv_j]=la_dur_bck[lv_j];	// Create backup.
			la_rate[lv_j]=la_rate_bck[lv_j];	// Create backup.
			la_nrs[lv_j]=la_nrs_bck[lv_j];	// Create backup.
			la_priority[lv_j]=la_priority_bck[lv_j];	// Create backup.
		}
		for(lv_c=0;lv_c<lv_nrp_SS;lv_c++)
		{
			lm_SS_pairs[lv_c][0]=lm_SS_pairs_bck[lv_c][0];	// Create backup.
			lm_SS_pairs[lv_c][1]=lm_SS_pairs_bck[lv_c][1];	// Create backup.
		}

		#pragma endregion

	}
	else if(lv_redundant_suc==true)	// The topological order is ok, however, there are redundant successors.
	{

		#pragma region "RESTORE BACKUP"

		for(lv_j=0;lv_j<lv_nra;lv_j++)
		{
			for(lv_s=0;lv_s<la_nrs[lv_j];lv_s++)
			{
				lm_suc[lv_j][lv_s]=lm_suc_bck[lv_j][lv_s];	// Create backup.
			}
			la_nrs[lv_j]=la_nrs_bck[lv_j];	// Create backup.
		}

		#pragma endregion

	}

	#pragma endregion

	#pragma endregion

	cout << "\n" << "Press any key to quit...";
	cin >> lv_project;

	#pragma region "[STEP 6] DESTRUCTION PHASE (AFTER LAST FUNCTION CALL)"

	#pragma region "DESTROY STRUCTURES CREATED IN STEP 4"

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		delete lm_suc[lv_j];
	}
	delete lm_suc;
	delete la_nrs;

	for(lv_c=0;lv_c<lv_nrp_SS;lv_c++)
	{
		delete lm_SS_pairs[lv_c];
	}
	delete lm_SS_pairs;

	delete la_priority;

	#pragma endregion

	#pragma region "DESTROY STRUCTURES CREATED IN STEP 3"

	delete la_queue;

	for(lv_x=0;lv_x<2;lv_x++)
	{
		for(lv_z=0;lv_z<lv_mxz;lv_z++)
		{
			delete lm_state_rxa[lv_x][lv_z];
			delete lm_state_bin_fin[lv_x][lv_z];
			delete lm_state_bin_idl[lv_x][lv_z];
			delete lm_state_bus[lv_x][lv_z];
			delete lm_state_eli[lv_x][lv_z];
			delete lm_state_eli_act[lv_x][lv_z];
		}
		delete lm_state_rxa[lv_x];
		delete lm_state_bin_fin[lv_x];
		delete lm_state_bin_idl[lv_x];
		delete lm_state_bus[lv_x];
		delete lm_state_eli[lv_x];
		delete lm_state_eli_act[lv_x];
		delete lm_state_makespan[lv_x];
		delete lm_state_prob[lv_x];
		delete lm_state_tot_rate[lv_x];
		delete lm_state_nrb[lv_x];
		delete lm_state_nre[lv_x];
	}
	delete lm_state_rxa;
	delete lm_state_bin_fin;
	delete lm_state_bin_idl;
	delete lm_state_bus;
	delete lm_state_eli;
	delete lm_state_eli_act;
	delete lm_state_makespan;
	delete lm_state_prob;
	delete lm_state_tot_rate;
	delete lm_state_nrb;
	delete lm_state_nre;
	delete la_state_nrz;

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		delete lm_bin_tpre[lv_j];
		delete lm_pre[lv_j];
		delete lm_suc_ranked[lv_j];
		delete lm_tpre[lv_j];
	}
	delete lm_bin_tpre;
	delete lm_pre;
	delete lm_suc_ranked;
	delete lm_tpre;
	delete la_nrp;
	delete la_nrtp;

	delete la_act_priority;
	delete la_bin_fin;

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		delete lm_aSS_bin[lv_j];
		delete lm_aSS_mem[lv_j];
	}
	delete lm_aSS_bin;
	delete lm_aSS_mem;
	delete la_aSS_nrm;

	#pragma endregion

	#pragma region "DESTROY STRUCTURES CREATED IN STEP 3b"

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		delete lm_rxu_bck[lv_j];
		delete lm_suc_bck[lv_j];
		delete lm_suc_red[lv_j];
	}
	delete lm_rxu_bck;
	delete lm_suc_bck;
	delete lm_suc_red;
	for(lv_c=0;lv_c<lv_mxp_SS;lv_c++)
	{
		delete lm_SS_pairs_bck[lv_c];
	}
	delete la_dur_bck;
	delete la_rate_bck;
	delete la_act2pos;
	delete la_nrs_bck;
	delete la_pos2act;
	delete la_priority_bck;
	
	#pragma endregion

	#pragma region "DESTROY STRUCTURES CREATED IN STEP 2"

	delete la_rate;

	for(lv_i=0;lv_i<=lv_mxb_size;lv_i++)
	{
		delete lm_bin_act[lv_i];
	}
	delete lm_bin_act;
	delete la_bin;
	delete la_bin_act;
	delete la_bin_nra;
	delete la_bin_val_act;

	#pragma endregion

	#pragma region "DESTROY STRUCTURES CREATED IN STEP 0"

	for(lv_j=0;lv_j<lv_nra;lv_j++)
	{
		delete lm_rxu[lv_j];
		delete lm_suc_ini[lv_j];
	}
	delete lm_rxu;
	delete lm_suc_ini;
	delete la_dur;
	delete la_rxa;

	#pragma endregion

	#pragma endregion

}